/*     */ package net.rubyeye.xmemcached.transcoders;
/*     */ 
/*     */ import java.io.ByteArrayInputStream;
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.ObjectInputStream;
/*     */ import java.io.ObjectOutputStream;
/*     */ import java.io.UnsupportedEncodingException;
/*     */ import java.util.zip.DeflaterOutputStream;
/*     */ import java.util.zip.GZIPInputStream;
/*     */ import java.util.zip.GZIPOutputStream;
/*     */ import java.util.zip.InflaterInputStream;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class BaseSerializingTranscoder
/*     */ {
/*     */   public static final int DEFAULT_COMPRESSION_THRESHOLD = 16384;
/*     */   public static final String DEFAULT_CHARSET = "UTF-8";
/*  30 */   protected int compressionThreshold = 16384;
/*  31 */   protected String charset = "UTF-8";
/*  32 */   protected CompressionMode compressMode = CompressionMode.GZIP;
/*  33 */   protected static final Logger log = LoggerFactory.getLogger(BaseSerializingTranscoder.class);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void setCompressionThreshold(int to)
/*     */   {
/*  45 */     this.compressionThreshold = to;
/*     */   }
/*     */   
/*     */   public CompressionMode getCompressMode() {
/*  49 */     return this.compressMode;
/*     */   }
/*     */   
/*     */   public void setCompressionMode(CompressionMode compressMode) {
/*  53 */     this.compressMode = compressMode;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   public void setCharset(String to)
/*     */   {
/*     */     try
/*     */     {
/*  62 */       new String(new byte[97], to);
/*     */     } catch (UnsupportedEncodingException e) {
/*  64 */       throw new RuntimeException(e);
/*     */     }
/*  66 */     this.charset = to;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected byte[] serialize(Object o)
/*     */   {
/*  73 */     if (o == null) {
/*  74 */       throw new NullPointerException("Can't serialize null");
/*     */     }
/*  76 */     byte[] rv = null;
/*     */     try {
/*  78 */       ByteArrayOutputStream bos = new ByteArrayOutputStream();
/*  79 */       ObjectOutputStream os = new ObjectOutputStream(bos);
/*  80 */       os.writeObject(o);
/*  81 */       os.close();
/*  82 */       bos.close();
/*  83 */       rv = bos.toByteArray();
/*     */     } catch (IOException e) {
/*  85 */       throw new IllegalArgumentException("Non-serializable object", e);
/*     */     }
/*  87 */     return rv;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected Object deserialize(byte[] in)
/*     */   {
/*  94 */     rv = null;
/*  95 */     ByteArrayInputStream bis = null;
/*  96 */     ObjectInputStream is = null;
/*     */     try {
/*  98 */       if (in != null) {
/*  99 */         bis = new ByteArrayInputStream(in);
/* 100 */         is = new ObjectInputStream(bis); }
/* 101 */       return is.readObject();
/*     */     }
/*     */     catch (IOException e)
/*     */     {
/* 105 */       log.error("Caught IOException decoding " + in.length + " bytes of data", e);
/*     */     }
/*     */     catch (ClassNotFoundException e) {
/* 108 */       log.error("Caught CNFE decoding " + in.length + " bytes of data", e);
/*     */     } finally {
/* 110 */       if (is != null) {
/*     */         try {
/* 112 */           is.close();
/*     */         }
/*     */         catch (IOException e) {}
/*     */       }
/*     */       
/* 117 */       if (bis != null) {
/*     */         try {
/* 119 */           bis.close();
/*     */         }
/*     */         catch (IOException e) {}
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final byte[] compress(byte[] in)
/*     */   {
/* 132 */     switch (this.compressMode) {
/*     */     case GZIP: 
/* 134 */       return gzipCompress(in);
/*     */     case ZIP: 
/* 136 */       return zipCompress(in);
/*     */     }
/* 138 */     return gzipCompress(in);
/*     */   }
/*     */   
/*     */ 
/*     */   private byte[] zipCompress(byte[] in)
/*     */   {
/* 144 */     baos = new ByteArrayOutputStream(in.length);
/* 145 */     DeflaterOutputStream os = new DeflaterOutputStream(baos);
/*     */     try {
/* 147 */       os.write(in);
/* 148 */       os.finish();
/*     */       try {
/* 150 */         os.close();
/*     */       } catch (IOException e) {
/* 152 */         log.error("Close DeflaterOutputStream error", e);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 163 */       return baos.toByteArray();
/*     */     }
/*     */     catch (IOException e)
/*     */     {
/* 155 */       throw new RuntimeException("IO exception compressing data", e);
/*     */     } finally {
/*     */       try {
/* 158 */         baos.close();
/*     */       } catch (IOException e) {
/* 160 */         log.error("Close ByteArrayOutputStream error", e);
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */   private static byte[] gzipCompress(byte[] in)
/*     */   {
/* 167 */     if (in == null) {
/* 168 */       throw new NullPointerException("Can't compress null");
/*     */     }
/* 170 */     ByteArrayOutputStream bos = new ByteArrayOutputStream();
/* 171 */     GZIPOutputStream gz = null;
/*     */     try {
/* 173 */       gz = new GZIPOutputStream(bos);
/* 174 */       gz.write(in);
/*     */       
/*     */ 
/*     */ 
/* 178 */       if (gz != null) {
/*     */         try {
/* 180 */           gz.close();
/*     */         } catch (IOException e) {
/* 182 */           log.error("Close GZIPOutputStream error", e);
/*     */         }
/*     */       }
/* 185 */       if (bos != null) {
/*     */         try {
/* 187 */           bos.close();
/*     */         } catch (IOException e) {
/* 189 */           log.error("Close ByteArrayOutputStream error", e);
/*     */         }
/*     */       }
/*     */       
/* 193 */       rv = bos.toByteArray();
/*     */     }
/*     */     catch (IOException e)
/*     */     {
/* 176 */       throw new RuntimeException("IO exception compressing data", e);
/*     */     } finally {
/* 178 */       if (gz != null) {
/*     */         try {
/* 180 */           gz.close();
/*     */         } catch (IOException e) {
/* 182 */           log.error("Close GZIPOutputStream error", e);
/*     */         }
/*     */       }
/* 185 */       if (bos != null) {
/*     */         try {
/* 187 */           bos.close();
/*     */         } catch (IOException e) {
/* 189 */           log.error("Close ByteArrayOutputStream error", e);
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */     byte[] rv;
/* 195 */     return rv;
/*     */   }
/*     */   
/* 198 */   static int COMPRESS_RATIO = 8;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected byte[] decompress(byte[] in)
/*     */   {
/* 206 */     switch (this.compressMode) {
/*     */     case GZIP: 
/* 208 */       return gzipDecompress(in);
/*     */     case ZIP: 
/* 210 */       return zipDecompress(in);
/*     */     }
/* 212 */     return gzipDecompress(in);
/*     */   }
/*     */   
/*     */   private byte[] zipDecompress(byte[] in)
/*     */   {
/* 217 */     int size = in.length * COMPRESS_RATIO;
/* 218 */     ByteArrayInputStream bais = new ByteArrayInputStream(in);
/* 219 */     InflaterInputStream is = new InflaterInputStream(bais);
/* 220 */     ByteArrayOutputStream baos = new ByteArrayOutputStream(size);
/*     */     try {
/* 222 */       byte[] uncompressMessage = new byte[size];
/*     */       int len;
/* 224 */       for (;;) { len = is.read(uncompressMessage);
/* 225 */         if (len <= 0) {
/*     */           break;
/*     */         }
/* 228 */         baos.write(uncompressMessage, 0, len);
/*     */       }
/* 230 */       baos.flush();
/* 231 */       return baos.toByteArray();
/*     */     }
/*     */     catch (IOException e) {
/* 234 */       log.error("Failed to decompress data", e);
/* 235 */       baos = null;
/*     */     } finally {
/*     */       try {
/* 238 */         is.close();
/*     */       } catch (IOException e) {
/* 240 */         log.error("failed to close InflaterInputStream");
/*     */       }
/*     */       try {
/* 243 */         bais.close();
/*     */       } catch (IOException e) {
/* 245 */         log.error("failed to close ByteArrayInputStream");
/*     */       }
/*     */       try {
/* 248 */         baos.close();
/*     */       } catch (IOException e) {
/* 250 */         log.error("failed to close ByteArrayOutputStream");
/*     */       }
/*     */     }
/* 253 */     return baos == null ? null : baos.toByteArray();
/*     */   }
/*     */   
/*     */   private byte[] gzipDecompress(byte[] in) {
/* 257 */     ByteArrayOutputStream bos = null;
/* 258 */     if (in != null) {
/* 259 */       ByteArrayInputStream bis = new ByteArrayInputStream(in);
/* 260 */       bos = new ByteArrayOutputStream();
/* 261 */       GZIPInputStream gis = null;
/*     */       try {
/* 263 */         gis = new GZIPInputStream(bis);
/*     */         
/* 265 */         byte[] buf = new byte['䀀'];
/* 266 */         int r = -1;
/* 267 */         while ((r = gis.read(buf)) > 0) {
/* 268 */           bos.write(buf, 0, r);
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 274 */         if (gis != null) {
/*     */           try {
/* 276 */             gis.close();
/*     */           } catch (IOException e) {
/* 278 */             log.error("Close GZIPInputStream error", e);
/*     */           }
/*     */         }
/* 281 */         if (bis != null) {
/*     */           try {
/* 283 */             bis.close();
/*     */           } catch (IOException e) {
/* 285 */             log.error("Close ByteArrayInputStream error", e);
/*     */           }
/*     */         }
/*     */         
/*     */ 
/* 290 */         if (bos != null) {
/*     */           break label262;
/*     */         }
/*     */       }
/*     */       catch (IOException e)
/*     */       {
/* 271 */         log.error("Failed to decompress data", e);
/* 272 */         bos = null;
/*     */       } finally {
/* 274 */         if (gis != null) {
/*     */           try {
/* 276 */             gis.close();
/*     */           } catch (IOException e) {
/* 278 */             log.error("Close GZIPInputStream error", e);
/*     */           }
/*     */         }
/* 281 */         if (bis != null) {
/*     */           try {
/* 283 */             bis.close();
/*     */           } catch (IOException e) {
/* 285 */             log.error("Close ByteArrayInputStream error", e);
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 290 */     tmpTernaryOp = null; label262: return bos.toByteArray();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected String decodeString(byte[] data)
/*     */   {
/* 297 */     String rv = null;
/*     */     try {
/* 299 */       if (data != null) {
/* 300 */         rv = new String(data, this.charset);
/*     */       }
/*     */     } catch (UnsupportedEncodingException e) {
/* 303 */       throw new RuntimeException(e);
/*     */     }
/* 305 */     return rv;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected byte[] encodeString(String in)
/*     */   {
/* 312 */     byte[] rv = null;
/*     */     try {
/* 314 */       rv = in.getBytes(this.charset);
/*     */     } catch (UnsupportedEncodingException e) {
/* 316 */       throw new RuntimeException(e);
/*     */     }
/* 318 */     return rv;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\net\rubyeye\xmemcached\transcoders\BaseSerializingTranscoder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */