/*     */ package org.apache.avro.io;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.util.List;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.Schema.Field.Order;
/*     */ import org.apache.avro.generic.GenericDatumReader;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class BinaryData
/*     */ {
/*     */   private static class Decoders
/*     */   {
/*     */     private final BinaryDecoder.BufferAccessor b1;
/*     */     private final BinaryDecoder.BufferAccessor b2;
/*     */     private final BinaryDecoder d1;
/*     */     private final BinaryDecoder d2;
/*     */     
/*     */     public Decoders()
/*     */     {
/*  37 */       this.d1 = new BinaryDecoder(new byte[0], 0, 0);
/*  38 */       this.d2 = new BinaryDecoder(new byte[0], 0, 0);
/*  39 */       this.b1 = this.d1.getBufferAccessor();
/*  40 */       this.b2 = this.d2.getBufferAccessor();
/*     */     }
/*     */     
/*     */     public void set(byte[] data1, int off1, int len1, byte[] data2, int off2, int len2) {
/*  44 */       this.d1.configure(data1, off1, len1);
/*  45 */       this.d2.configure(data2, off2, len2);
/*     */     }
/*     */   }
/*     */   
/*  49 */   private static final ThreadLocal<Decoders> DECODERS = new ThreadLocal() {
/*     */     protected BinaryData.Decoders initialValue() {
/*  51 */       return new BinaryData.Decoders();
/*     */     }
/*     */   };
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int compare(byte[] b1, int s1, byte[] b2, int s2, Schema schema)
/*     */   {
/*  60 */     return compare(b1, s1, b1.length - s1, b2, s2, b2.length - s2, schema);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int compare(byte[] b1, int s1, int l1, byte[] b2, int s2, int l2, Schema schema)
/*     */   {
/*  69 */     Decoders decoders = (Decoders)DECODERS.get();
/*  70 */     decoders.set(b1, s1, l1, b2, s2, l2);
/*     */     try {
/*  72 */       return compare(decoders, schema);
/*     */     } catch (IOException e) {
/*  74 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private static int compare(Decoders d, Schema schema)
/*     */     throws IOException
/*     */   {
/*  81 */     Decoder d1 = d.d1;Decoder d2 = d.d2;
/*  82 */     switch (schema.getType()) {
/*     */     case RECORD: 
/*  84 */       for (Schema.Field field : schema.getFields())
/*  85 */         if (field.order() == Schema.Field.Order.IGNORE) {
/*  86 */           GenericDatumReader.skip(field.schema(), d1);
/*  87 */           GenericDatumReader.skip(field.schema(), d2);
/*     */         }
/*     */         else {
/*  90 */           int c = compare(d, field.schema());
/*  91 */           if (c != 0)
/*  92 */             return field.order() != Schema.Field.Order.DESCENDING ? c : -c;
/*     */         }
/*  94 */       return 0;
/*     */     case ENUM: 
/*     */     case INT: 
/*  97 */       int i1 = d1.readInt();
/*  98 */       int i2 = d2.readInt();
/*  99 */       return i1 > i2 ? 1 : i1 == i2 ? 0 : -1;
/*     */     
/*     */     case LONG: 
/* 102 */       long l1 = d1.readLong();
/* 103 */       long l2 = d2.readLong();
/* 104 */       return l1 > l2 ? 1 : l1 == l2 ? 0 : -1;
/*     */     
/*     */     case ARRAY: 
/* 107 */       long i = 0L;
/* 108 */       long r1 = 0L;long r2 = 0L;
/* 109 */       long l1 = 0L;long l2 = 0L;
/*     */       for (;;) {
/* 111 */         if (r1 == 0L) {
/* 112 */           r1 = d1.readLong();
/* 113 */           if (r1 < 0L) { r1 = -r1;d1.readLong(); }
/* 114 */           l1 += r1;
/*     */         }
/* 116 */         if (r2 == 0L) {
/* 117 */           r2 = d2.readLong();
/* 118 */           if (r2 < 0L) { r2 = -r2;d2.readLong(); }
/* 119 */           l2 += r2;
/*     */         }
/* 121 */         if ((r1 == 0L) || (r2 == 0L))
/* 122 */           return l1 > l2 ? 1 : l1 == l2 ? 0 : -1;
/* 123 */         long l = Math.min(l1, l2);
/* 124 */         for (; i < l; 
/*     */             
/*     */ 
/* 127 */             r2 -= 1L)
/*     */         {
/* 125 */           int c = compare(d, schema.getElementType());
/* 126 */           if (c != 0) return c;
/* 127 */           i += 1L;r1 -= 1L;
/*     */         }
/*     */       }
/*     */     
/*     */     case MAP: 
/* 132 */       throw new AvroRuntimeException("Can't compare maps!");
/*     */     case UNION: 
/* 134 */       int i1 = d1.readInt();
/* 135 */       int i2 = d2.readInt();
/* 136 */       if (i1 == i2) {
/* 137 */         return compare(d, (Schema)schema.getTypes().get(i1));
/*     */       }
/* 139 */       return i1 - i2;
/*     */     
/*     */ 
/*     */     case FIXED: 
/* 143 */       int size = schema.getFixedSize();
/* 144 */       int c = compareBytes(d.b1.getBuf(), d.b1.getPos(), size, d.b2.getBuf(), d.b2.getPos(), size);
/*     */       
/* 146 */       d.d1.skipFixed(size);
/* 147 */       d.d2.skipFixed(size);
/* 148 */       return c;
/*     */     case STRING: 
/*     */     case BYTES: 
/* 151 */       int l1 = d1.readInt();
/* 152 */       int l2 = d2.readInt();
/* 153 */       int c = compareBytes(d.b1.getBuf(), d.b1.getPos(), l1, d.b2.getBuf(), d.b2.getPos(), l2);
/*     */       
/* 155 */       d.d1.skipFixed(l1);
/* 156 */       d.d2.skipFixed(l2);
/* 157 */       return c;
/*     */     
/*     */     case FLOAT: 
/* 160 */       float f1 = d1.readFloat();
/* 161 */       float f2 = d2.readFloat();
/* 162 */       return f1 > f2 ? 1 : f1 == f2 ? 0 : -1;
/*     */     
/*     */     case DOUBLE: 
/* 165 */       double f1 = d1.readDouble();
/* 166 */       double f2 = d2.readDouble();
/* 167 */       return f1 > f2 ? 1 : f1 == f2 ? 0 : -1;
/*     */     
/*     */     case BOOLEAN: 
/* 170 */       boolean b1 = d1.readBoolean();
/* 171 */       boolean b2 = d2.readBoolean();
/* 172 */       return b1 ? 1 : b1 == b2 ? 0 : -1;
/*     */     case NULL: 
/* 174 */       return 0;
/*     */     }
/* 176 */     throw new AvroRuntimeException("Unexpected schema to compare!");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int compareBytes(byte[] b1, int s1, int l1, byte[] b2, int s2, int l2)
/*     */   {
/* 184 */     int end1 = s1 + l1;
/* 185 */     int end2 = s2 + l2;
/* 186 */     int i = s1; for (int j = s2; (i < end1) && (j < end2); j++) {
/* 187 */       int a = b1[i] & 0xFF;
/* 188 */       int b = b2[j] & 0xFF;
/* 189 */       if (a != b) {
/* 190 */         return a - b;
/*     */       }
/* 186 */       i++;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 193 */     return l1 - l2;
/*     */   }
/*     */   
/*     */   private static class HashData {
/*     */     private final BinaryDecoder.BufferAccessor bytes;
/*     */     private final BinaryDecoder decoder;
/*     */     
/* 200 */     public HashData() { this.decoder = new BinaryDecoder(new byte[0], 0, 0);
/* 201 */       this.bytes = this.decoder.getBufferAccessor();
/*     */     }
/*     */     
/* 204 */     public void set(byte[] bytes, int start, int len) { this.decoder.configure(bytes, start, len); }
/*     */   }
/*     */   
/*     */ 
/* 208 */   private static final ThreadLocal<HashData> HASH_DATA = new ThreadLocal() {
/*     */     protected BinaryData.HashData initialValue() {
/* 210 */       return new BinaryData.HashData();
/*     */     }
/*     */   };
/*     */   
/*     */ 
/*     */   public static int hashCode(byte[] bytes, int start, int length, Schema schema)
/*     */   {
/* 217 */     HashData data = (HashData)HASH_DATA.get();
/* 218 */     data.set(bytes, start, length);
/*     */     try {
/* 220 */       return hashCode(data, schema);
/*     */     } catch (IOException e) {
/* 222 */       throw new AvroRuntimeException(e);
/*     */     }
/*     */   }
/*     */   
/*     */   private static int hashCode(HashData data, Schema schema) throws IOException
/*     */   {
/* 228 */     Decoder decoder = data.decoder;
/* 229 */     switch (schema.getType()) {
/*     */     case RECORD: 
/* 231 */       int hashCode = 1;
/* 232 */       for (Schema.Field field : schema.getFields()) {
/* 233 */         if (field.order() == Schema.Field.Order.IGNORE) {
/* 234 */           GenericDatumReader.skip(field.schema(), decoder);
/*     */         }
/*     */         else
/* 237 */           hashCode = hashCode * 31 + hashCode(data, field.schema());
/*     */       }
/* 239 */       return hashCode;
/*     */     case ENUM: 
/*     */     case INT: 
/* 242 */       return decoder.readInt();
/*     */     case FLOAT: 
/* 244 */       return Float.floatToIntBits(decoder.readFloat());
/*     */     case LONG: 
/* 246 */       long l = decoder.readLong();
/* 247 */       return (int)(l ^ l >>> 32);
/*     */     
/*     */     case DOUBLE: 
/* 250 */       long l = Double.doubleToLongBits(decoder.readDouble());
/* 251 */       return (int)(l ^ l >>> 32);
/*     */     
/*     */     case ARRAY: 
/* 254 */       Schema elementType = schema.getElementType();
/* 255 */       int hashCode = 1;
/* 256 */       for (long l = decoder.readArrayStart(); l != 0L; l = decoder.arrayNext())
/* 257 */         for (long i = 0L; i < l; i += 1L)
/* 258 */           hashCode = hashCode * 31 + hashCode(data, elementType);
/* 259 */       return hashCode;
/*     */     
/*     */     case MAP: 
/* 262 */       throw new AvroRuntimeException("Can't hashCode maps!");
/*     */     case UNION: 
/* 264 */       return hashCode(data, (Schema)schema.getTypes().get(decoder.readInt()));
/*     */     case FIXED: 
/* 266 */       return hashBytes(1, data, schema.getFixedSize(), false);
/*     */     case STRING: 
/* 268 */       return hashBytes(0, data, decoder.readInt(), false);
/*     */     case BYTES: 
/* 270 */       return hashBytes(1, data, decoder.readInt(), true);
/*     */     case BOOLEAN: 
/* 272 */       return decoder.readBoolean() ? 1231 : 1237;
/*     */     case NULL: 
/* 274 */       return 0;
/*     */     }
/* 276 */     throw new AvroRuntimeException("Unexpected schema to hashCode!");
/*     */   }
/*     */   
/*     */   private static int hashBytes(int init, HashData data, int len, boolean rev)
/*     */     throws IOException
/*     */   {
/* 282 */     int hashCode = init;
/* 283 */     byte[] bytes = data.bytes.getBuf();
/* 284 */     int start = data.bytes.getPos();
/* 285 */     int end = start + len;
/* 286 */     if (rev) {
/* 287 */       for (int i = end - 1; i >= start; i--)
/* 288 */         hashCode = hashCode * 31 + bytes[i];
/*     */     } else
/* 290 */       for (int i = start; i < end; i++)
/* 291 */         hashCode = hashCode * 31 + bytes[i];
/* 292 */     data.decoder.skipFixed(len);
/* 293 */     return hashCode;
/*     */   }
/*     */   
/*     */   public static int skipLong(byte[] bytes, int start)
/*     */   {
/* 298 */     int i = start;
/* 299 */     for (int b = bytes[(i++)]; (b & 0x80) != 0; b = bytes[(i++)]) {}
/* 300 */     return i;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int encodeBoolean(boolean b, byte[] buf, int pos)
/*     */   {
/* 308 */     buf[pos] = (b ? 1 : 0);
/* 309 */     return 1;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int encodeInt(int n, byte[] buf, int pos)
/*     */   {
/* 319 */     n = n << 1 ^ n >> 31;
/* 320 */     int start = pos;
/* 321 */     if ((n & 0xFFFFFF80) != 0) {
/* 322 */       buf[(pos++)] = ((byte)((n | 0x80) & 0xFF));
/* 323 */       n >>>= 7;
/* 324 */       if (n > 127) {
/* 325 */         buf[(pos++)] = ((byte)((n | 0x80) & 0xFF));
/* 326 */         n >>>= 7;
/* 327 */         if (n > 127) {
/* 328 */           buf[(pos++)] = ((byte)((n | 0x80) & 0xFF));
/* 329 */           n >>>= 7;
/* 330 */           if (n > 127) {
/* 331 */             buf[(pos++)] = ((byte)((n | 0x80) & 0xFF));
/* 332 */             n >>>= 7;
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 337 */     buf[(pos++)] = ((byte)n);
/* 338 */     return pos - start;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int encodeLong(long n, byte[] buf, int pos)
/*     */   {
/* 348 */     n = n << 1 ^ n >> 63;
/* 349 */     int start = pos;
/* 350 */     if ((n & 0xFFFFFFFFFFFFFF80) != 0L) {
/* 351 */       buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 352 */       n >>>= 7;
/* 353 */       if (n > 127L) {
/* 354 */         buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 355 */         n >>>= 7;
/* 356 */         if (n > 127L) {
/* 357 */           buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 358 */           n >>>= 7;
/* 359 */           if (n > 127L) {
/* 360 */             buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 361 */             n >>>= 7;
/* 362 */             if (n > 127L) {
/* 363 */               buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 364 */               n >>>= 7;
/* 365 */               if (n > 127L) {
/* 366 */                 buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 367 */                 n >>>= 7;
/* 368 */                 if (n > 127L) {
/* 369 */                   buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 370 */                   n >>>= 7;
/* 371 */                   if (n > 127L) {
/* 372 */                     buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 373 */                     n >>>= 7;
/* 374 */                     if (n > 127L) {
/* 375 */                       buf[(pos++)] = ((byte)(int)((n | 0x80) & 0xFF));
/* 376 */                       n >>>= 7;
/*     */                     }
/*     */                   }
/*     */                 }
/*     */               }
/*     */             }
/*     */           }
/*     */         }
/*     */       }
/*     */     }
/* 386 */     buf[(pos++)] = ((byte)(int)n);
/* 387 */     return pos - start;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int encodeFloat(float f, byte[] buf, int pos)
/*     */   {
/* 396 */     int len = 1;
/* 397 */     int bits = Float.floatToRawIntBits(f);
/*     */     
/* 399 */     buf[pos] = ((byte)(bits & 0xFF));
/* 400 */     buf[(pos + len++)] = ((byte)(bits >>> 8 & 0xFF));
/* 401 */     buf[(pos + len++)] = ((byte)(bits >>> 16 & 0xFF));
/* 402 */     buf[(pos + len++)] = ((byte)(bits >>> 24 & 0xFF));
/* 403 */     return 4;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static int encodeDouble(double d, byte[] buf, int pos)
/*     */   {
/* 412 */     long bits = Double.doubleToRawLongBits(d);
/* 413 */     int first = (int)(bits & 0xFFFFFFFFFFFFFFFF);
/* 414 */     int second = (int)(bits >>> 32 & 0xFFFFFFFFFFFFFFFF);
/*     */     
/*     */ 
/* 417 */     buf[pos] = ((byte)(first & 0xFF));
/* 418 */     buf[(pos + 4)] = ((byte)(second & 0xFF));
/* 419 */     buf[(pos + 5)] = ((byte)(second >>> 8 & 0xFF));
/* 420 */     buf[(pos + 1)] = ((byte)(first >>> 8 & 0xFF));
/* 421 */     buf[(pos + 2)] = ((byte)(first >>> 16 & 0xFF));
/* 422 */     buf[(pos + 6)] = ((byte)(second >>> 16 & 0xFF));
/* 423 */     buf[(pos + 7)] = ((byte)(second >>> 24 & 0xFF));
/* 424 */     buf[(pos + 3)] = ((byte)(first >>> 24 & 0xFF));
/* 425 */     return 8;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\io\BinaryData.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */