/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.lang.reflect.Method;
/*     */ import java.math.BigDecimal;
/*     */ import java.net.InetAddress;
/*     */ import java.sql.Timestamp;
/*     */ import java.util.Calendar;
/*     */ import java.util.Collection;
/*     */ import java.util.EnumMap;
/*     */ import java.util.EnumSet;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.Map.Entry;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.BeanProperty;
/*     */ import org.codehaus.jackson.map.ContextualSerializer;
/*     */ import org.codehaus.jackson.map.JsonMappingException;
/*     */ import org.codehaus.jackson.map.JsonSerializable;
/*     */ import org.codehaus.jackson.map.JsonSerializableWithType;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.JsonSerializer.None;
/*     */ import org.codehaus.jackson.map.SerializationConfig;
/*     */ import org.codehaus.jackson.map.SerializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.SerializerFactory;
/*     */ import org.codehaus.jackson.map.Serializers;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.annotate.JsonSerialize.Typing;
/*     */ import org.codehaus.jackson.map.ext.OptionalHandlerFactory;
/*     */ import org.codehaus.jackson.map.introspect.Annotated;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*     */ import org.codehaus.jackson.map.jsontype.SubtypeResolver;
/*     */ import org.codehaus.jackson.map.jsontype.TypeResolverBuilder;
/*     */ import org.codehaus.jackson.map.ser.impl.InetAddressSerializer;
/*     */ import org.codehaus.jackson.map.ser.impl.ObjectArraySerializer;
/*     */ import org.codehaus.jackson.map.ser.impl.StringCollectionSerializer;
/*     */ import org.codehaus.jackson.map.type.ArrayType;
/*     */ import org.codehaus.jackson.map.type.CollectionLikeType;
/*     */ import org.codehaus.jackson.map.type.CollectionType;
/*     */ import org.codehaus.jackson.map.type.MapLikeType;
/*     */ import org.codehaus.jackson.map.type.MapType;
/*     */ import org.codehaus.jackson.map.type.TypeFactory;
/*     */ import org.codehaus.jackson.map.util.ClassUtil;
/*     */ import org.codehaus.jackson.map.util.EnumValues;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ import org.codehaus.jackson.util.TokenBuffer;
/*     */ 
/*     */ public abstract class BasicSerializerFactory extends SerializerFactory
/*     */ {
/*  50 */   protected static final HashMap<String, JsonSerializer<?>> _concrete = new HashMap();
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  60 */   protected static final HashMap<String, Class<? extends JsonSerializer<?>>> _concreteLazy = new HashMap();
/*     */   
/*     */   protected static final HashMap<String, JsonSerializer<?>> _arraySerializers;
/*     */   
/*     */ 
/*     */   static
/*     */   {
/*  67 */     _concrete.put(String.class.getName(), new StdSerializers.StringSerializer());
/*  68 */     ToStringSerializer sls = ToStringSerializer.instance;
/*  69 */     _concrete.put(StringBuffer.class.getName(), sls);
/*  70 */     _concrete.put(StringBuilder.class.getName(), sls);
/*  71 */     _concrete.put(Character.class.getName(), sls);
/*  72 */     _concrete.put(Character.TYPE.getName(), sls);
/*     */     
/*     */ 
/*  75 */     _concrete.put(Boolean.TYPE.getName(), new StdSerializers.BooleanSerializer(true));
/*  76 */     _concrete.put(Boolean.class.getName(), new StdSerializers.BooleanSerializer(false));
/*  77 */     JsonSerializer<?> intS = new StdSerializers.IntegerSerializer();
/*  78 */     _concrete.put(Integer.class.getName(), intS);
/*  79 */     _concrete.put(Integer.TYPE.getName(), intS);
/*  80 */     _concrete.put(Long.class.getName(), StdSerializers.LongSerializer.instance);
/*  81 */     _concrete.put(Long.TYPE.getName(), StdSerializers.LongSerializer.instance);
/*  82 */     _concrete.put(Byte.class.getName(), StdSerializers.IntLikeSerializer.instance);
/*  83 */     _concrete.put(Byte.TYPE.getName(), StdSerializers.IntLikeSerializer.instance);
/*  84 */     _concrete.put(Short.class.getName(), StdSerializers.IntLikeSerializer.instance);
/*  85 */     _concrete.put(Short.TYPE.getName(), StdSerializers.IntLikeSerializer.instance);
/*     */     
/*     */ 
/*  88 */     _concrete.put(Float.class.getName(), StdSerializers.FloatSerializer.instance);
/*  89 */     _concrete.put(Float.TYPE.getName(), StdSerializers.FloatSerializer.instance);
/*  90 */     _concrete.put(Double.class.getName(), StdSerializers.DoubleSerializer.instance);
/*  91 */     _concrete.put(Double.TYPE.getName(), StdSerializers.DoubleSerializer.instance);
/*     */     
/*     */ 
/*  94 */     JsonSerializer<?> ns = new StdSerializers.NumberSerializer();
/*  95 */     _concrete.put(java.math.BigInteger.class.getName(), ns);
/*  96 */     _concrete.put(BigDecimal.class.getName(), ns);
/*     */     
/*     */ 
/*     */ 
/* 100 */     _concrete.put(Calendar.class.getName(), StdSerializers.CalendarSerializer.instance);
/* 101 */     _concrete.put(java.util.Date.class.getName(), StdSerializers.UtilDateSerializer.instance);
/* 102 */     _concrete.put(java.sql.Date.class.getName(), new StdSerializers.SqlDateSerializer());
/* 103 */     _concrete.put(java.sql.Time.class.getName(), new StdSerializers.SqlTimeSerializer());
/*     */     
/* 105 */     _concrete.put(Timestamp.class.getName(), StdSerializers.UtilDateSerializer.instance);
/*     */     
/*     */ 
/* 108 */     for (Map.Entry<Class<?>, Object> en : new JdkSerializers().provide()) {
/* 109 */       Object value = en.getValue();
/* 110 */       if ((value instanceof JsonSerializer)) {
/* 111 */         _concrete.put(((Class)en.getKey()).getName(), (JsonSerializer)value);
/* 112 */       } else if ((value instanceof Class))
/*     */       {
/* 114 */         Class<? extends JsonSerializer<?>> cls = (Class)value;
/* 115 */         _concreteLazy.put(((Class)en.getKey()).getName(), cls);
/*     */       } else {
/* 117 */         throw new IllegalStateException("Internal error: unrecognized value of type " + en.getClass().getName());
/*     */       }
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 123 */     _concreteLazy.put(TokenBuffer.class.getName(), StdSerializers.TokenBufferSerializer.class);
/*     */     
/*     */ 
/* 126 */     _arraySerializers = new HashMap();
/*     */     
/*     */ 
/*     */ 
/* 130 */     _arraySerializers.put(boolean[].class.getName(), new ArraySerializers.BooleanArraySerializer());
/* 131 */     _arraySerializers.put(byte[].class.getName(), new ArraySerializers.ByteArraySerializer());
/* 132 */     _arraySerializers.put(char[].class.getName(), new ArraySerializers.CharArraySerializer());
/* 133 */     _arraySerializers.put(short[].class.getName(), new ArraySerializers.ShortArraySerializer());
/* 134 */     _arraySerializers.put(int[].class.getName(), new ArraySerializers.IntArraySerializer());
/* 135 */     _arraySerializers.put(long[].class.getName(), new ArraySerializers.LongArraySerializer());
/* 136 */     _arraySerializers.put(float[].class.getName(), new ArraySerializers.FloatArraySerializer());
/* 137 */     _arraySerializers.put(double[].class.getName(), new ArraySerializers.DoubleArraySerializer());
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/* 144 */   protected OptionalHandlerFactory optionalHandlers = OptionalHandlerFactory.instance;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public TypeSerializer createTypeSerializer(SerializationConfig config, JavaType baseType, BeanProperty property)
/*     */   {
/* 180 */     BasicBeanDescription bean = (BasicBeanDescription)config.introspectClassAnnotations(baseType.getRawClass());
/* 181 */     org.codehaus.jackson.map.introspect.AnnotatedClass ac = bean.getClassInfo();
/* 182 */     AnnotationIntrospector ai = config.getAnnotationIntrospector();
/* 183 */     TypeResolverBuilder<?> b = ai.findTypeResolver(config, ac, baseType);
/*     */     
/*     */ 
/*     */ 
/* 187 */     Collection<org.codehaus.jackson.map.jsontype.NamedType> subtypes = null;
/* 188 */     if (b == null) {
/* 189 */       b = config.getDefaultTyper(baseType);
/*     */     } else {
/* 191 */       subtypes = config.getSubtypeResolver().collectAndResolveSubtypes(ac, config, ai);
/*     */     }
/* 193 */     return b == null ? null : b.buildTypeSerializer(config, baseType, subtypes, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final JsonSerializer<?> getNullSerializer()
/*     */   {
/* 204 */     return NullSerializer.instance;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final JsonSerializer<?> findSerializerByLookup(JavaType type, SerializationConfig config, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping)
/*     */   {
/* 223 */     Class<?> raw = type.getRawClass();
/* 224 */     String clsName = raw.getName();
/* 225 */     JsonSerializer<?> ser = (JsonSerializer)_concrete.get(clsName);
/* 226 */     if (ser != null) {
/* 227 */       return ser;
/*     */     }
/* 229 */     Class<? extends JsonSerializer<?>> serClass = (Class)_concreteLazy.get(clsName);
/* 230 */     if (serClass != null) {
/*     */       try {
/* 232 */         return (JsonSerializer)serClass.newInstance();
/*     */       } catch (Exception e) {
/* 234 */         throw new IllegalStateException("Failed to instantiate standard serializer (of type " + serClass.getName() + "): " + e.getMessage(), e);
/*     */       }
/*     */     }
/*     */     
/* 238 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final JsonSerializer<?> findSerializerByPrimaryType(JavaType type, SerializationConfig config, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping)
/*     */     throws JsonMappingException
/*     */   {
/* 254 */     Class<?> raw = type.getRawClass();
/*     */     
/* 256 */     if (JsonSerializable.class.isAssignableFrom(raw)) {
/* 257 */       if (JsonSerializableWithType.class.isAssignableFrom(raw)) {
/* 258 */         return StdSerializers.SerializableWithTypeSerializer.instance;
/*     */       }
/* 260 */       return StdSerializers.SerializableSerializer.instance;
/*     */     }
/*     */     
/* 263 */     AnnotatedMethod valueMethod = beanDesc.findJsonValueMethod();
/* 264 */     if (valueMethod != null)
/*     */     {
/* 266 */       Method m = valueMethod.getAnnotated();
/* 267 */       if (config.isEnabled(SerializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 268 */         ClassUtil.checkAndFixAccess(m);
/*     */       }
/* 270 */       JsonSerializer<Object> ser = findSerializerFromAnnotation(config, valueMethod, property);
/* 271 */       return new JsonValueSerializer(m, ser, property);
/*     */     }
/*     */     
/*     */ 
/* 275 */     if (InetAddress.class.isAssignableFrom(raw)) {
/* 276 */       return InetAddressSerializer.instance;
/*     */     }
/*     */     
/* 279 */     if (java.util.TimeZone.class.isAssignableFrom(raw)) {
/* 280 */       return org.codehaus.jackson.map.ser.impl.TimeZoneSerializer.instance;
/*     */     }
/*     */     
/*     */ 
/* 284 */     JsonSerializer<?> ser = this.optionalHandlers.findSerializer(config, type);
/* 285 */     if (ser != null) {
/* 286 */       return ser;
/*     */     }
/*     */     
/* 289 */     if (Number.class.isAssignableFrom(raw)) {
/* 290 */       return StdSerializers.NumberSerializer.instance;
/*     */     }
/* 292 */     if (Enum.class.isAssignableFrom(raw))
/*     */     {
/* 294 */       Class<Enum<?>> enumClass = raw;
/* 295 */       return EnumSerializer.construct(enumClass, config, beanDesc);
/*     */     }
/* 297 */     if (Calendar.class.isAssignableFrom(raw)) {
/* 298 */       return StdSerializers.CalendarSerializer.instance;
/*     */     }
/* 300 */     if (java.util.Date.class.isAssignableFrom(raw)) {
/* 301 */       return StdSerializers.UtilDateSerializer.instance;
/*     */     }
/* 303 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public final JsonSerializer<?> findSerializerByAddonType(SerializationConfig config, JavaType javaType, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping)
/*     */     throws JsonMappingException
/*     */   {
/* 319 */     Class<?> type = javaType.getRawClass();
/*     */     
/*     */ 
/* 322 */     if (Iterator.class.isAssignableFrom(type)) {
/* 323 */       return buildIteratorSerializer(config, javaType, beanDesc, property, staticTyping);
/*     */     }
/* 325 */     if (Iterable.class.isAssignableFrom(type)) {
/* 326 */       return buildIterableSerializer(config, javaType, beanDesc, property, staticTyping);
/*     */     }
/* 328 */     if (CharSequence.class.isAssignableFrom(type)) {
/* 329 */       return ToStringSerializer.instance;
/*     */     }
/* 331 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<Object> findSerializerFromAnnotation(SerializationConfig config, Annotated a, BeanProperty property)
/*     */     throws JsonMappingException
/*     */   {
/* 346 */     Object serDef = config.getAnnotationIntrospector().findSerializer(a);
/* 347 */     if (serDef == null) {
/* 348 */       return null;
/*     */     }
/* 350 */     if ((serDef instanceof JsonSerializer)) {
/* 351 */       JsonSerializer<Object> ser = (JsonSerializer)serDef;
/* 352 */       if ((ser instanceof ContextualSerializer)) {
/* 353 */         return ((ContextualSerializer)ser).createContextual(config, property);
/*     */       }
/* 355 */       return ser;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 360 */     if (!(serDef instanceof Class)) {
/* 361 */       throw new IllegalStateException("AnnotationIntrospector returned value of type " + serDef.getClass().getName() + "; expected type JsonSerializer or Class<JsonSerializer> instead");
/*     */     }
/* 363 */     Class<?> cls = (Class)serDef;
/* 364 */     if (!JsonSerializer.class.isAssignableFrom(cls)) {
/* 365 */       throw new IllegalStateException("AnnotationIntrospector returned Class " + cls.getName() + "; expected Class<JsonSerializer>");
/*     */     }
/* 367 */     JsonSerializer<Object> ser = config.serializerInstance(a, cls);
/* 368 */     if ((ser instanceof ContextualSerializer)) {
/* 369 */       return ((ContextualSerializer)ser).createContextual(config, property);
/*     */     }
/* 371 */     return ser;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public JsonSerializer<?> buildContainerSerializer(SerializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping)
/*     */   {
/* 384 */     JavaType elementType = type.getContentType();
/* 385 */     TypeSerializer elementTypeSerializer = createTypeSerializer(config, elementType, property);
/*     */     
/*     */ 
/* 388 */     if (elementTypeSerializer != null) {
/* 389 */       staticTyping = false;
/* 390 */     } else if (!staticTyping) {
/* 391 */       staticTyping = usesStaticTyping(config, beanDesc, elementTypeSerializer, property);
/*     */     }
/* 393 */     JsonSerializer<Object> elementValueSerializer = findContentSerializer(config, beanDesc.getClassInfo(), property);
/*     */     
/*     */ 
/* 396 */     if (type.isMapLikeType()) {
/* 397 */       MapLikeType mlt = (MapLikeType)type;
/* 398 */       JsonSerializer<Object> keySerializer = findKeySerializer(config, beanDesc.getClassInfo(), property);
/* 399 */       if (mlt.isTrueMapType()) {
/* 400 */         return buildMapSerializer(config, (MapType)mlt, beanDesc, property, staticTyping, keySerializer, elementTypeSerializer, elementValueSerializer);
/*     */       }
/*     */       
/* 403 */       return buildMapLikeSerializer(config, mlt, beanDesc, property, staticTyping, keySerializer, elementTypeSerializer, elementValueSerializer);
/*     */     }
/*     */     
/* 406 */     if (type.isCollectionLikeType()) {
/* 407 */       CollectionLikeType clt = (CollectionLikeType)type;
/* 408 */       if (clt.isTrueCollectionType()) {
/* 409 */         return buildCollectionSerializer(config, (CollectionType)clt, beanDesc, property, staticTyping, elementTypeSerializer, elementValueSerializer);
/*     */       }
/*     */       
/* 412 */       return buildCollectionLikeSerializer(config, clt, beanDesc, property, staticTyping, elementTypeSerializer, elementValueSerializer);
/*     */     }
/*     */     
/* 415 */     if (type.isArrayType()) {
/* 416 */       return buildArraySerializer(config, (ArrayType)type, beanDesc, property, staticTyping, elementTypeSerializer, elementValueSerializer);
/*     */     }
/*     */     
/* 419 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildCollectionLikeSerializer(SerializationConfig config, CollectionLikeType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 434 */     for (Serializers serializers : customSerializers()) {
/* 435 */       JsonSerializer<?> ser = serializers.findCollectionLikeSerializer(config, type, beanDesc, property, elementTypeSerializer, elementValueSerializer);
/*     */       
/* 437 */       if (ser != null) {
/* 438 */         return ser;
/*     */       }
/*     */     }
/* 441 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildCollectionSerializer(SerializationConfig config, CollectionType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 457 */     for (Serializers serializers : customSerializers()) {
/* 458 */       JsonSerializer<?> ser = serializers.findCollectionSerializer(config, type, beanDesc, property, elementTypeSerializer, elementValueSerializer);
/*     */       
/* 460 */       if (ser != null) {
/* 461 */         return ser;
/*     */       }
/*     */     }
/* 464 */     Class<?> raw = type.getRawClass();
/* 465 */     if (EnumSet.class.isAssignableFrom(raw)) {
/* 466 */       return buildEnumSetSerializer(config, type, beanDesc, property, staticTyping, elementTypeSerializer, elementValueSerializer);
/*     */     }
/*     */     
/* 469 */     Class<?> elementRaw = type.getContentType().getRawClass();
/* 470 */     if (isIndexedList(raw)) {
/* 471 */       if (elementRaw == String.class) {
/* 472 */         return new org.codehaus.jackson.map.ser.impl.IndexedStringListSerializer(property);
/*     */       }
/* 474 */       return ContainerSerializers.indexedListSerializer(type.getContentType(), staticTyping, elementTypeSerializer, property, elementValueSerializer);
/*     */     }
/*     */     
/* 477 */     if (elementRaw == String.class) {
/* 478 */       return new StringCollectionSerializer(property);
/*     */     }
/* 480 */     return ContainerSerializers.collectionSerializer(type.getContentType(), staticTyping, elementTypeSerializer, property, elementValueSerializer);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildEnumSetSerializer(SerializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 490 */     JavaType enumType = type.getContentType();
/*     */     
/* 492 */     if (!enumType.isEnumType()) {
/* 493 */       enumType = null;
/*     */     }
/* 495 */     return ContainerSerializers.enumSetSerializer(enumType, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean isIndexedList(Class<?> cls)
/*     */   {
/* 503 */     return java.util.RandomAccess.class.isAssignableFrom(cls);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildMapLikeSerializer(SerializationConfig config, MapLikeType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, JsonSerializer<Object> keySerializer, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 525 */     for (Serializers serializers : customSerializers()) {
/* 526 */       JsonSerializer<?> ser = serializers.findMapLikeSerializer(config, type, beanDesc, property, keySerializer, elementTypeSerializer, elementValueSerializer);
/*     */       
/* 528 */       if (ser != null) {
/* 529 */         return ser;
/*     */       }
/*     */     }
/* 532 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildMapSerializer(SerializationConfig config, MapType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, JsonSerializer<Object> keySerializer, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 547 */     for (Serializers serializers : customSerializers()) {
/* 548 */       JsonSerializer<?> ser = serializers.findMapSerializer(config, type, beanDesc, property, keySerializer, elementTypeSerializer, elementValueSerializer);
/*     */       
/* 550 */       if (ser != null) {
/* 551 */         return ser;
/*     */       }
/*     */     }
/* 554 */     if (EnumMap.class.isAssignableFrom(type.getRawClass())) {
/* 555 */       return buildEnumMapSerializer(config, type, beanDesc, property, staticTyping, elementTypeSerializer, elementValueSerializer);
/*     */     }
/*     */     
/* 558 */     return MapSerializer.construct(config.getAnnotationIntrospector().findPropertiesToIgnore(beanDesc.getClassInfo()), type, staticTyping, elementTypeSerializer, property, keySerializer, elementValueSerializer);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildEnumMapSerializer(SerializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 574 */     JavaType keyType = type.getKeyType();
/*     */     
/* 576 */     EnumValues enums = null;
/* 577 */     if (keyType.isEnumType())
/*     */     {
/* 579 */       Class<Enum<?>> enumClass = keyType.getRawClass();
/* 580 */       enums = EnumValues.construct(enumClass, config.getAnnotationIntrospector());
/*     */     }
/* 582 */     return new EnumMapSerializer(type.getContentType(), staticTyping, enums, elementTypeSerializer, property, elementValueSerializer);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildArraySerializer(SerializationConfig config, ArrayType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping, TypeSerializer elementTypeSerializer, JsonSerializer<Object> elementValueSerializer)
/*     */   {
/* 601 */     Class<?> raw = type.getRawClass();
/* 602 */     if (String[].class == raw) {
/* 603 */       return new ArraySerializers.StringArraySerializer(property);
/*     */     }
/*     */     
/* 606 */     JsonSerializer<?> ser = (JsonSerializer)_arraySerializers.get(raw.getName());
/* 607 */     if (ser != null) {
/* 608 */       return ser;
/*     */     }
/* 610 */     return new ObjectArraySerializer(type.getContentType(), staticTyping, elementTypeSerializer, property, elementValueSerializer);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildIteratorSerializer(SerializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping)
/*     */   {
/* 625 */     JavaType valueType = type.containedType(0);
/* 626 */     if (valueType == null) {
/* 627 */       valueType = TypeFactory.unknownType();
/*     */     }
/* 629 */     TypeSerializer vts = createTypeSerializer(config, valueType, property);
/* 630 */     return ContainerSerializers.iteratorSerializer(valueType, usesStaticTyping(config, beanDesc, vts, property), vts, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JsonSerializer<?> buildIterableSerializer(SerializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property, boolean staticTyping)
/*     */   {
/* 639 */     JavaType valueType = type.containedType(0);
/* 640 */     if (valueType == null) {
/* 641 */       valueType = TypeFactory.unknownType();
/*     */     }
/* 643 */     TypeSerializer vts = createTypeSerializer(config, valueType, property);
/* 644 */     return ContainerSerializers.iterableSerializer(valueType, usesStaticTyping(config, beanDesc, vts, property), vts, property);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected <T extends JavaType> T modifyTypeByAnnotation(SerializationConfig config, Annotated a, T type)
/*     */   {
/* 664 */     Class<?> superclass = config.getAnnotationIntrospector().findSerializationType(a);
/* 665 */     if (superclass != null) {
/*     */       try {
/* 667 */         type = type.widenBy(superclass);
/*     */       } catch (IllegalArgumentException iae) {
/* 669 */         throw new IllegalArgumentException("Failed to widen type " + type + " with concrete-type annotation (value " + superclass.getName() + "), method '" + a.getName() + "': " + iae.getMessage());
/*     */       }
/*     */     }
/* 672 */     return modifySecondaryTypesByAnnotation(config, a, type);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static <T extends JavaType> T modifySecondaryTypesByAnnotation(SerializationConfig config, Annotated a, T type)
/*     */   {
/* 681 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/*     */     
/* 683 */     if (type.isContainerType()) {
/* 684 */       Class<?> keyClass = intr.findSerializationKeyType(a, type.getKeyType());
/* 685 */       if (keyClass != null)
/*     */       {
/* 687 */         if (!(type instanceof MapType)) {
/* 688 */           throw new IllegalArgumentException("Illegal key-type annotation: type " + type + " is not a Map type");
/*     */         }
/*     */         try {
/* 691 */           type = ((MapType)type).widenKey(keyClass);
/*     */         } catch (IllegalArgumentException iae) {
/* 693 */           throw new IllegalArgumentException("Failed to narrow key type " + type + " with key-type annotation (" + keyClass.getName() + "): " + iae.getMessage());
/*     */         }
/*     */       }
/*     */       
/*     */ 
/* 698 */       Class<?> cc = intr.findSerializationContentType(a, type.getContentType());
/* 699 */       if (cc != null) {
/*     */         try {
/* 701 */           type = type.widenContentsBy(cc);
/*     */         } catch (IllegalArgumentException iae) {
/* 703 */           throw new IllegalArgumentException("Failed to narrow content type " + type + " with content-type annotation (" + cc.getName() + "): " + iae.getMessage());
/*     */         }
/*     */       }
/*     */     }
/* 707 */     return type;
/*     */   }
/*     */   
/*     */ 
/*     */   protected static JsonSerializer<Object> findKeySerializer(SerializationConfig config, Annotated a, BeanProperty property)
/*     */   {
/* 713 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 714 */     Class<? extends JsonSerializer<?>> serClass = intr.findKeySerializer(a);
/* 715 */     if (((serClass == null) || (serClass == JsonSerializer.None.class)) && 
/* 716 */       (property != null)) {
/* 717 */       serClass = intr.findKeySerializer(property.getMember());
/*     */     }
/*     */     
/* 720 */     if ((serClass != null) && (serClass != JsonSerializer.None.class)) {
/* 721 */       return config.serializerInstance(a, serClass);
/*     */     }
/* 723 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */   protected static JsonSerializer<Object> findContentSerializer(SerializationConfig config, Annotated a, BeanProperty property)
/*     */   {
/* 729 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 730 */     Class<? extends JsonSerializer<?>> serClass = intr.findContentSerializer(a);
/* 731 */     if (((serClass == null) || (serClass == JsonSerializer.None.class)) && 
/* 732 */       (property != null)) {
/* 733 */       serClass = intr.findContentSerializer(property.getMember());
/*     */     }
/*     */     
/* 736 */     if ((serClass != null) && (serClass != JsonSerializer.None.class)) {
/* 737 */       return config.serializerInstance(a, serClass);
/*     */     }
/* 739 */     return null;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected boolean usesStaticTyping(SerializationConfig config, BasicBeanDescription beanDesc, TypeSerializer typeSer, BeanProperty property)
/*     */   {
/* 754 */     if (typeSer != null) {
/* 755 */       return false;
/*     */     }
/* 757 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/* 758 */     JsonSerialize.Typing t = intr.findSerializationTyping(beanDesc.getClassInfo());
/* 759 */     if (t != null) {
/* 760 */       if (t == JsonSerialize.Typing.STATIC) {
/* 761 */         return true;
/*     */       }
/*     */     }
/* 764 */     else if (config.isEnabled(SerializationConfig.Feature.USE_STATIC_TYPING)) {
/* 765 */       return true;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 771 */     if (property != null) {
/* 772 */       JavaType type = property.getType();
/* 773 */       if (type.isContainerType()) {
/* 774 */         if (intr.findSerializationContentType(property.getMember(), property.getType()) != null) {
/* 775 */           return true;
/*     */         }
/* 777 */         if (((type instanceof MapType)) && 
/* 778 */           (intr.findSerializationKeyType(property.getMember(), property.getType()) != null)) {
/* 779 */           return true;
/*     */         }
/*     */       }
/*     */     }
/*     */     
/* 784 */     return false;
/*     */   }
/*     */   
/*     */   public abstract JsonSerializer<Object> createSerializer(SerializationConfig paramSerializationConfig, JavaType paramJavaType, BeanProperty paramBeanProperty)
/*     */     throws JsonMappingException;
/*     */   
/*     */   protected abstract Iterable<Serializers> customSerializers();
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\BasicSerializerFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */