/*     */ package org.codehaus.jackson.map.ser;
/*     */ 
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.Method;
/*     */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*     */ import org.codehaus.jackson.map.JsonSerializer;
/*     */ import org.codehaus.jackson.map.SerializationConfig;
/*     */ import org.codehaus.jackson.map.SerializationConfig.Feature;
/*     */ import org.codehaus.jackson.map.TypeSerializer;
/*     */ import org.codehaus.jackson.map.annotate.JsonSerialize.Inclusion;
/*     */ import org.codehaus.jackson.map.annotate.JsonSerialize.Typing;
/*     */ import org.codehaus.jackson.map.introspect.Annotated;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedClass;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedField;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMember;
/*     */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*     */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*     */ import org.codehaus.jackson.map.util.Annotations;
/*     */ import org.codehaus.jackson.type.JavaType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class PropertyBuilder
/*     */ {
/*     */   protected final SerializationConfig _config;
/*     */   protected final BasicBeanDescription _beanDesc;
/*     */   protected final JsonSerialize.Inclusion _outputProps;
/*     */   protected final AnnotationIntrospector _annotationIntrospector;
/*     */   protected Object _defaultBean;
/*     */   
/*     */   public PropertyBuilder(SerializationConfig config, BasicBeanDescription beanDesc)
/*     */   {
/*  39 */     this._config = config;
/*  40 */     this._beanDesc = beanDesc;
/*  41 */     this._outputProps = beanDesc.findSerializationInclusion(config.getSerializationInclusion());
/*  42 */     this._annotationIntrospector = this._config.getAnnotationIntrospector();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public Annotations getClassAnnotations()
/*     */   {
/*  52 */     return this._beanDesc.getClassAnnotations();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   protected BeanPropertyWriter buildWriter(String name, JavaType declaredType, JsonSerializer<Object> ser, TypeSerializer typeSer, TypeSerializer contentTypeSer, AnnotatedMember am, boolean defaultUseStaticTyping)
/*     */   {
/*     */     Field f;
/*     */     
/*     */ 
/*     */     Method m;
/*     */     
/*     */ 
/*     */     Field f;
/*     */     
/*  67 */     if ((am instanceof AnnotatedField)) {
/*  68 */       Method m = null;
/*  69 */       f = ((AnnotatedField)am).getAnnotated();
/*     */     } else {
/*  71 */       m = ((AnnotatedMethod)am).getAnnotated();
/*  72 */       f = null;
/*     */     }
/*     */     
/*     */ 
/*  76 */     JavaType serializationType = findSerializationType(am, defaultUseStaticTyping, declaredType);
/*     */     
/*     */ 
/*  79 */     if (contentTypeSer != null)
/*     */     {
/*     */ 
/*     */ 
/*     */ 
/*  84 */       if (serializationType == null)
/*     */       {
/*  86 */         serializationType = declaredType;
/*     */       }
/*  88 */       JavaType ct = serializationType.getContentType();
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*  93 */       if (ct == null) {
/*  94 */         throw new IllegalStateException("Problem trying to create BeanPropertyWriter for property '" + name + "' (of type " + this._beanDesc.getType() + "); serialization type " + serializationType + " has no content");
/*     */       }
/*     */       
/*  97 */       serializationType = serializationType.withContentTypeHandler(contentTypeSer);
/*  98 */       ct = serializationType.getContentType();
/*     */     }
/*     */     
/* 101 */     Object suppValue = null;
/* 102 */     boolean suppressNulls = false;
/*     */     
/* 104 */     JsonSerialize.Inclusion methodProps = this._annotationIntrospector.findSerializationInclusion(am, this._outputProps);
/*     */     
/* 106 */     if (methodProps != null) {
/* 107 */       switch (methodProps) {
/*     */       case NON_DEFAULT: 
/* 109 */         suppValue = getDefaultValue(name, m, f);
/* 110 */         if (suppValue == null) {
/* 111 */           suppressNulls = true;
/*     */         }
/*     */         break;
/*     */       case NON_NULL: 
/* 115 */         suppressNulls = true;
/*     */       }
/*     */       
/*     */     }
/* 119 */     return new BeanPropertyWriter(am, this._beanDesc.getClassAnnotations(), name, declaredType, ser, typeSer, serializationType, m, f, suppressNulls, suppValue);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected JavaType findSerializationType(Annotated a, boolean useStaticTyping, JavaType declaredType)
/*     */   {
/* 138 */     Class<?> serClass = this._annotationIntrospector.findSerializationType(a);
/* 139 */     if (serClass != null)
/*     */     {
/* 141 */       Class<?> rawDeclared = declaredType.getRawClass();
/* 142 */       if (serClass.isAssignableFrom(rawDeclared)) {
/* 143 */         declaredType = declaredType.widenBy(serClass);
/*     */ 
/*     */ 
/*     */       }
/*     */       else
/*     */       {
/*     */ 
/*     */ 
/* 151 */         if (!rawDeclared.isAssignableFrom(serClass)) {
/* 152 */           throw new IllegalArgumentException("Illegal concrete-type annotation for method '" + a.getName() + "': class " + serClass.getName() + " not a super-type of (declared) class " + rawDeclared.getName());
/*     */         }
/*     */         
/*     */ 
/*     */ 
/*     */ 
/* 158 */         declaredType = declaredType.forcedNarrowBy(serClass);
/*     */       }
/* 160 */       useStaticTyping = true;
/*     */     }
/*     */     
/* 163 */     JavaType secondary = BeanSerializerFactory.modifySecondaryTypesByAnnotation(this._config, a, declaredType);
/* 164 */     if (secondary != declaredType) {
/* 165 */       useStaticTyping = true;
/* 166 */       declaredType = secondary;
/*     */     }
/*     */     
/*     */ 
/*     */ 
/*     */ 
/* 172 */     if (!useStaticTyping) {
/* 173 */       JsonSerialize.Typing typing = this._annotationIntrospector.findSerializationTyping(a);
/* 174 */       if (typing != null) {
/* 175 */         useStaticTyping = typing == JsonSerialize.Typing.STATIC;
/*     */       }
/*     */     }
/* 178 */     return useStaticTyping ? declaredType : null;
/*     */   }
/*     */   
/*     */   protected Object getDefaultBean()
/*     */   {
/* 183 */     if (this._defaultBean == null)
/*     */     {
/*     */ 
/*     */ 
/* 187 */       this._defaultBean = this._beanDesc.instantiateBean(this._config.isEnabled(SerializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS));
/* 188 */       if (this._defaultBean == null) {
/* 189 */         Class<?> cls = this._beanDesc.getClassInfo().getAnnotated();
/* 190 */         throw new IllegalArgumentException("Class " + cls.getName() + " has no default constructor; can not instantiate default bean value to support 'properties=JsonSerialize.Inclusion.NON_DEFAULT' annotation");
/*     */       }
/*     */     }
/* 193 */     return this._defaultBean;
/*     */   }
/*     */   
/*     */   protected Object getDefaultValue(String name, Method m, Field f)
/*     */   {
/* 198 */     Object defaultBean = getDefaultBean();
/*     */     try {
/* 200 */       if (m != null) {
/* 201 */         return m.invoke(defaultBean, new Object[0]);
/*     */       }
/* 203 */       return f.get(defaultBean);
/*     */     } catch (Exception e) {
/* 205 */       return _throwWrapped(e, name, defaultBean);
/*     */     }
/*     */   }
/*     */   
/*     */   protected Object _throwWrapped(Exception e, String propName, Object defaultBean)
/*     */   {
/* 211 */     Throwable t = e;
/* 212 */     while (t.getCause() != null) {
/* 213 */       t = t.getCause();
/*     */     }
/* 215 */     if ((t instanceof Error)) throw ((Error)t);
/* 216 */     if ((t instanceof RuntimeException)) throw ((RuntimeException)t);
/* 217 */     throw new IllegalArgumentException("Failed to get property '" + propName + "' of default " + defaultBean.getClass().getName() + " instance");
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\ser\PropertyBuilder.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */