/*     */ package org.hibernate.annotations.common.reflection.java.generics;
/*     */ 
/*     */ import java.lang.reflect.Array;
/*     */ import java.lang.reflect.GenericArrayType;
/*     */ import java.lang.reflect.ParameterizedType;
/*     */ import java.lang.reflect.Type;
/*     */ import java.lang.reflect.TypeVariable;
/*     */ import java.lang.reflect.WildcardType;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ class ApproximatingTypeEnvironment
/*     */   implements TypeEnvironment
/*     */ {
/*     */   public Type bind(Type type)
/*     */   {
/*  43 */     Type result = fineApproximation(type);
/*  44 */     assert (TypeUtils.isResolved(result));
/*  45 */     return result;
/*     */   }
/*     */   
/*     */   private Type fineApproximation(Type type) {
/*  49 */     (Type)new TypeSwitch() {
/*     */       public Type caseWildcardType(WildcardType wildcardType) {
/*  51 */         return wildcardType;
/*     */       }
/*     */       
/*     */       public Type caseClass(Class classType)
/*     */       {
/*  56 */         return classType;
/*     */       }
/*     */       
/*     */       public Type caseGenericArrayType(GenericArrayType genericArrayType)
/*     */       {
/*  61 */         if (TypeUtils.isResolved(genericArrayType)) {
/*  62 */           return genericArrayType;
/*     */         }
/*  64 */         Type componentType = genericArrayType.getGenericComponentType();
/*  65 */         Type boundComponentType = ApproximatingTypeEnvironment.this.bind(componentType);
/*  66 */         if ((boundComponentType instanceof Class)) {
/*  67 */           return Array.newInstance((Class)boundComponentType, 0).getClass();
/*     */         }
/*     */         
/*     */ 
/*  71 */         return Object[].class;
/*     */       }
/*     */       
/*     */       public Type caseParameterizedType(ParameterizedType parameterizedType)
/*     */       {
/*  76 */         if (TypeUtils.isResolved(parameterizedType)) {
/*  77 */           return parameterizedType;
/*     */         }
/*     */         
/*  80 */         if (!TypeUtils.isCollection(parameterizedType)) {
/*  81 */           return Object.class;
/*     */         }
/*     */         
/*  84 */         Type[] typeArguments = parameterizedType.getActualTypeArguments();
/*  85 */         Type[] approximatedTypeArguments = new Type[typeArguments.length];
/*  86 */         for (int i = 0; i < typeArguments.length; i++) {
/*  87 */           approximatedTypeArguments[i] = ApproximatingTypeEnvironment.this.coarseApproximation(typeArguments[i]);
/*     */         }
/*     */         
/*  90 */         return TypeFactory.createParameterizedType(ApproximatingTypeEnvironment.this.bind(parameterizedType.getRawType()), approximatedTypeArguments, parameterizedType.getOwnerType());
/*     */       }
/*     */       
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*  99 */       public Type defaultCase(Type t) { return ApproximatingTypeEnvironment.this.coarseApproximation(t); } }.doSwitch(type);
/*     */   }
/*     */   
/*     */ 
/*     */   private Type coarseApproximation(Type type)
/*     */   {
/* 105 */     Type result = (Type)new TypeSwitch() {
/*     */       public Type caseWildcardType(WildcardType wildcardType) {
/* 107 */         return approximateTo(wildcardType.getUpperBounds());
/*     */       }
/*     */       
/*     */       public Type caseGenericArrayType(GenericArrayType genericArrayType)
/*     */       {
/* 112 */         if (TypeUtils.isResolved(genericArrayType)) {
/* 113 */           return genericArrayType;
/*     */         }
/* 115 */         return Object[].class;
/*     */       }
/*     */       
/*     */       public Type caseParameterizedType(ParameterizedType parameterizedType)
/*     */       {
/* 120 */         if (TypeUtils.isResolved(parameterizedType)) {
/* 121 */           return parameterizedType;
/*     */         }
/* 123 */         return Object.class;
/*     */       }
/*     */       
/*     */       public Type caseTypeVariable(TypeVariable typeVariable)
/*     */       {
/* 128 */         return approximateTo(typeVariable.getBounds());
/*     */       }
/*     */       
/*     */       private Type approximateTo(Type[] bounds) {
/* 132 */         if (bounds.length != 1) {
/* 133 */           return Object.class;
/*     */         }
/* 135 */         return ApproximatingTypeEnvironment.this.coarseApproximation(bounds[0]);
/*     */       }
/*     */       
/*     */ 
/*     */ 
/* 140 */       public Type defaultCase(Type t) { return t; } }.doSwitch(type);
/*     */     
/*     */ 
/* 143 */     assert (TypeUtils.isResolved(result));
/* 144 */     return result;
/*     */   }
/*     */   
/*     */   public String toString()
/*     */   {
/* 149 */     return "approximated_types";
/*     */   }
/*     */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/reflection/java/generics/ApproximatingTypeEnvironment.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */