package prefuse.util.display;

import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.util.logging.Logger;

public class Clip
{
  private static final byte EMPTY = 0;
  private static final byte INUSE = 1;
  private static final byte INVALID = 2;
  private double[] clip = new double[8];
  private byte status = 2;
  
  public void reset()
  {
    this.status = 0;
  }
  
  public void invalidate()
  {
    this.status = 2;
  }
  
  public void setClip(Clip paramClip)
  {
    this.status = 1;
    System.arraycopy(paramClip.clip, 0, this.clip, 0, this.clip.length);
  }
  
  public void setClip(Rectangle2D paramRectangle2D)
  {
    setClip(paramRectangle2D.getX(), paramRectangle2D.getY(), paramRectangle2D.getWidth(), paramRectangle2D.getHeight());
  }
  
  public void setClip(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    this.status = 1;
    this.clip[0] = paramDouble1;
    this.clip[1] = paramDouble2;
    this.clip[6] = (paramDouble1 + paramDouble3);
    this.clip[7] = (paramDouble2 + paramDouble4);
  }
  
  public void transform(AffineTransform paramAffineTransform)
  {
    this.clip[2] = this.clip[0];
    this.clip[3] = this.clip[7];
    this.clip[4] = this.clip[6];
    this.clip[5] = this.clip[1];
    paramAffineTransform.transform(this.clip, 0, this.clip, 0, 4);
    double d1 = this.clip[0];
    double d2 = this.clip[1];
    double d3 = this.clip[6];
    double d4 = this.clip[7];
    for (int i = 0; i < 7; i += 2)
    {
      if (this.clip[i] < d1) {
        d1 = this.clip[i];
      }
      if (this.clip[i] > d3) {
        d3 = this.clip[i];
      }
      if (this.clip[(i + 1)] < d2) {
        d2 = this.clip[(i + 1)];
      }
      if (this.clip[(i + 1)] > d4) {
        d4 = this.clip[(i + 1)];
      }
    }
    this.clip[0] = d1;
    this.clip[1] = d2;
    this.clip[6] = d3;
    this.clip[7] = d4;
  }
  
  public void limit(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    this.clip[0] = Math.max(this.clip[0], paramDouble1);
    this.clip[1] = Math.max(this.clip[1], paramDouble2);
    this.clip[6] = Math.min(this.clip[6], paramDouble3);
    this.clip[7] = Math.min(this.clip[7], paramDouble4);
  }
  
  public boolean intersects(Rectangle2D paramRectangle2D, double paramDouble)
  {
    double d1 = this.clip[6] - this.clip[0];
    double d2 = this.clip[7] - this.clip[1];
    double d3 = paramRectangle2D.getWidth();
    double d4 = paramRectangle2D.getHeight();
    if ((d3 < 0.0D) || (d4 < 0.0D) || (d1 < 0.0D) || (d2 < 0.0D)) {
      return false;
    }
    double d5 = this.clip[0];
    double d6 = this.clip[1];
    double d7 = paramRectangle2D.getX() - paramDouble;
    double d8 = paramRectangle2D.getY() - paramDouble;
    d3 += d7 + 2.0D * paramDouble;
    d4 += d8 + 2.0D * paramDouble;
    d1 += d5;
    d2 += d6;
    return ((d3 < d7) || (d3 > d5)) && ((d4 < d8) || (d4 > d6)) && ((d1 < d5) || (d1 > d7)) && ((d2 < d6) || (d2 > d8));
  }
  
  public void union(Clip paramClip)
  {
    if (this.status == 2) {
      return;
    }
    if (this.status == 0)
    {
      setClip(paramClip);
      this.status = 1;
      return;
    }
    this.clip[0] = Math.min(this.clip[0], paramClip.clip[0]);
    this.clip[1] = Math.min(this.clip[1], paramClip.clip[1]);
    this.clip[6] = Math.max(this.clip[6], paramClip.clip[6]);
    this.clip[7] = Math.max(this.clip[7], paramClip.clip[7]);
  }
  
  public void union(Rectangle2D paramRectangle2D)
  {
    if (this.status == 2) {
      return;
    }
    double d1 = paramRectangle2D.getMinX();
    double d2 = paramRectangle2D.getMinY();
    double d3 = paramRectangle2D.getMaxX();
    double d4 = paramRectangle2D.getMaxY();
    if ((Double.isNaN(d1)) || (Double.isNaN(d2)) || (Double.isNaN(d3)) || (Double.isNaN(d4)))
    {
      Logger.getLogger(getClass().getName()).warning("Union with invalid clip region: " + paramRectangle2D);
      return;
    }
    if (this.status == 0)
    {
      setClip(paramRectangle2D);
      this.status = 1;
      return;
    }
    this.clip[0] = Math.min(this.clip[0], d1);
    this.clip[1] = Math.min(this.clip[1], d2);
    this.clip[6] = Math.max(this.clip[6], d3);
    this.clip[7] = Math.max(this.clip[7], d4);
  }
  
  public void union(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    if (this.status == 2) {
      return;
    }
    if (this.status == 0)
    {
      setClip(paramDouble1, paramDouble2, paramDouble3, paramDouble4);
      this.status = 1;
      return;
    }
    this.clip[0] = Math.min(this.clip[0], paramDouble1);
    this.clip[1] = Math.min(this.clip[1], paramDouble2);
    this.clip[6] = Math.max(this.clip[6], paramDouble1 + paramDouble3);
    this.clip[7] = Math.max(this.clip[7], paramDouble2 + paramDouble4);
  }
  
  public void intersection(Clip paramClip)
  {
    if (this.status == 2) {
      return;
    }
    if (this.status == 0)
    {
      setClip(paramClip);
      this.status = 1;
      return;
    }
    this.clip[0] = Math.max(this.clip[0], paramClip.clip[0]);
    this.clip[1] = Math.max(this.clip[1], paramClip.clip[1]);
    this.clip[6] = Math.min(this.clip[6], paramClip.clip[6]);
    this.clip[7] = Math.min(this.clip[7], paramClip.clip[7]);
  }
  
  public void intersection(Rectangle2D paramRectangle2D)
  {
    if (this.status == 2) {
      return;
    }
    if (this.status == 0)
    {
      setClip(paramRectangle2D);
      this.status = 1;
      return;
    }
    this.clip[0] = Math.max(this.clip[0], paramRectangle2D.getMinX());
    this.clip[1] = Math.max(this.clip[1], paramRectangle2D.getMinY());
    this.clip[6] = Math.min(this.clip[6], paramRectangle2D.getMaxX());
    this.clip[7] = Math.min(this.clip[7], paramRectangle2D.getMaxY());
  }
  
  public void intersection(double paramDouble1, double paramDouble2, double paramDouble3, double paramDouble4)
  {
    if (this.status == 2) {
      return;
    }
    if (this.status == 0)
    {
      setClip(paramDouble1, paramDouble2, paramDouble3, paramDouble4);
      this.status = 1;
      return;
    }
    this.clip[0] = Math.max(this.clip[0], paramDouble1);
    this.clip[1] = Math.max(this.clip[1], paramDouble2);
    this.clip[6] = Math.min(this.clip[6], paramDouble1 + paramDouble3);
    this.clip[7] = Math.min(this.clip[7], paramDouble2 + paramDouble4);
  }
  
  public void expandToIntegerLimits()
  {
    this.clip[0] = Math.floor(this.clip[0]);
    this.clip[1] = Math.floor(this.clip[1]);
    this.clip[6] = Math.ceil(this.clip[6]);
    this.clip[7] = Math.ceil(this.clip[7]);
  }
  
  public void expand(double paramDouble)
  {
    this.clip[0] -= paramDouble;
    this.clip[1] -= paramDouble;
    this.clip[6] += paramDouble;
    this.clip[7] += paramDouble;
  }
  
  public void grow(double paramDouble)
  {
    this.clip[6] += paramDouble;
    this.clip[7] += paramDouble;
  }
  
  public double getMinX()
  {
    return this.clip[0];
  }
  
  public double getMinY()
  {
    return this.clip[1];
  }
  
  public double getMaxX()
  {
    return this.clip[6];
  }
  
  public double getMaxY()
  {
    return this.clip[7];
  }
  
  public double getWidth()
  {
    return this.clip[6] - this.clip[0];
  }
  
  public double getHeight()
  {
    return this.clip[7] - this.clip[1];
  }
  
  public boolean isEmpty()
  {
    return this.status == 0;
  }
  
  public boolean isInvalid()
  {
    return this.status == 2;
  }
  
  public boolean equals(Object paramObject)
  {
    Object localObject;
    if ((paramObject instanceof Rectangle2D))
    {
      localObject = (Rectangle2D)paramObject;
      return (((Rectangle2D)localObject).getMinX() == this.clip[0]) && (((Rectangle2D)localObject).getMinY() == this.clip[1]) && (((Rectangle2D)localObject).getMaxX() == this.clip[6]) && (((Rectangle2D)localObject).getMaxY() == this.clip[7]);
    }
    if ((paramObject instanceof Clip))
    {
      localObject = (Clip)paramObject;
      if (((Clip)localObject).status == this.status)
      {
        if (this.status == 1) {
          return (localObject.clip[0] == this.clip[0]) && (localObject.clip[1] == this.clip[1]) && (localObject.clip[6] == this.clip[6]) && (localObject.clip[7] == this.clip[7]);
        }
        return true;
      }
      return false;
    }
    return false;
  }
  
  public String toString()
  {
    StringBuffer localStringBuffer = new StringBuffer(20);
    localStringBuffer.append("Clip[");
    switch (this.status)
    {
    case 2: 
      localStringBuffer.append("invalid");
      break;
    case 0: 
      localStringBuffer.append("empty");
      break;
    default: 
      localStringBuffer.append(this.clip[0]).append(",");
      localStringBuffer.append(this.clip[1]).append(",");
      localStringBuffer.append(this.clip[6]).append(",");
      localStringBuffer.append(this.clip[7]);
    }
    localStringBuffer.append("]");
    return localStringBuffer.toString();
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\display\Clip.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */