package prefuse.action.layout;

import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;
import java.util.Iterator;
import java.util.logging.Logger;
import prefuse.Visualization;
import prefuse.data.Schema;
import prefuse.data.query.ObjectRangeModel;
import prefuse.data.tuple.TupleSet;
import prefuse.data.util.Index;
import prefuse.util.MathLib;
import prefuse.util.PrefuseLib;
import prefuse.util.ui.ValuedRangeModel;
import prefuse.visual.VisualItem;
import prefuse.visual.VisualTable;

public class AxisLabelLayout
  extends Layout
{
  public static final String FRAC = "frac";
  public static final String LABEL = "_label";
  public static final String VALUE = "_value";
  private AxisLayout m_layout;
  private ValuedRangeModel m_model;
  private double m_lo;
  private double m_hi;
  private double m_prevlo;
  private double m_prevhi;
  private NumberFormat m_nf = NumberFormat.getInstance();
  private int m_axis;
  private int m_scale = 0;
  private double m_spacing;
  
  public AxisLabelLayout(String paramString, int paramInt, ValuedRangeModel paramValuedRangeModel)
  {
    this(paramString, paramInt, paramValuedRangeModel, null);
  }
  
  public AxisLabelLayout(String paramString, int paramInt, ValuedRangeModel paramValuedRangeModel, Rectangle2D paramRectangle2D)
  {
    super(paramString);
    if (paramRectangle2D != null) {
      super.setLayoutBounds(paramRectangle2D);
    }
    this.m_model = paramValuedRangeModel;
    this.m_axis = paramInt;
    this.m_spacing = 25.0D;
  }
  
  public AxisLabelLayout(String paramString, AxisLayout paramAxisLayout, Rectangle2D paramRectangle2D)
  {
    this(paramString, paramAxisLayout, paramRectangle2D, 25.0D);
  }
  
  public AxisLabelLayout(String paramString, AxisLayout paramAxisLayout, Rectangle2D paramRectangle2D, double paramDouble)
  {
    super(paramString);
    super.setLayoutBounds(paramRectangle2D);
    this.m_layout = paramAxisLayout;
    this.m_model = paramAxisLayout.getRangeModel();
    this.m_axis = paramAxisLayout.getAxis();
    this.m_scale = paramAxisLayout.getScale();
    this.m_spacing = paramDouble;
  }
  
  public NumberFormat getNumberFormat()
  {
    return this.m_nf;
  }
  
  public void setNumberFormat(NumberFormat paramNumberFormat)
  {
    this.m_nf = paramNumberFormat;
  }
  
  public double getSpacing()
  {
    return this.m_spacing;
  }
  
  public void setSpacing(double paramDouble)
  {
    this.m_spacing = paramDouble;
  }
  
  public int getScale()
  {
    return this.m_scale;
  }
  
  public void setScale(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 4)) {
      throw new IllegalArgumentException("Unrecognized scale type: " + paramInt);
    }
    this.m_scale = paramInt;
  }
  
  public void run(double paramDouble)
  {
    if ((this.m_model == null) && (this.m_layout != null)) {
      this.m_model = this.m_layout.getRangeModel();
    }
    if (this.m_model == null)
    {
      Logger.getLogger(getClass().getName()).warning("Axis labels missing a range model.");
      return;
    }
    VisualTable localVisualTable = getTable();
    Double localDouble = (Double)localVisualTable.getClientProperty("frac");
    double d = localDouble == null ? 1.0D : localDouble.doubleValue();
    this.m_prevlo += d * (this.m_lo - this.m_prevlo);
    this.m_prevhi += d * (this.m_hi - this.m_prevhi);
    if ((this.m_model instanceof ObjectRangeModel))
    {
      this.m_lo = this.m_model.getValue();
      this.m_hi = (this.m_lo + this.m_model.getExtent());
      ordinalLayout(localVisualTable);
    }
    else
    {
      this.m_lo = ((Number)this.m_model.getLowValue()).doubleValue();
      this.m_hi = ((Number)this.m_model.getHighValue()).doubleValue();
      switch (this.m_scale)
      {
      case 1: 
        logLayout(localVisualTable);
        break;
      case 2: 
        sqrtLayout(localVisualTable);
        break;
      case 0: 
      default: 
        linearLayout(localVisualTable);
      }
    }
    garbageCollect(localVisualTable);
  }
  
  protected void linearLayout(VisualTable paramVisualTable)
  {
    Rectangle2D localRectangle2D = getLayoutBounds();
    double d1 = getBreadth(localRectangle2D);
    double d2 = this.m_hi - this.m_lo;
    double d3 = this.m_prevhi - this.m_prevlo;
    double d4 = Math.pow(10.0D, Math.floor(MathLib.safeLog10(this.m_lo)));
    Iterator localIterator = paramVisualTable.tuples();
    while (localIterator.hasNext())
    {
      localObject = (VisualItem)localIterator.next();
      reset((VisualItem)localObject);
      d5 = ((VisualItem)localObject).getDouble("_value");
      double d6 = d2 == 0.0D ? 0.0D : (d5 - this.m_lo) / d2 * d1;
      set((VisualItem)localObject, d6, localRectangle2D);
    }
    Object localObject = paramVisualTable.index("_value");
    double d5 = getLinearStep(d2, d1 / d2);
    if (d5 == 0.0D) {
      d5 = 1.0D;
    }
    for (double d8 = d4; d8 <= this.m_hi; d8 += d5)
    {
      double d7 = (d8 - this.m_lo) / d2 * d1;
      if (d7 >= -0.5D)
      {
        int i;
        VisualItem localVisualItem;
        if ((i = ((Index)localObject).get(d8)) >= 0)
        {
          localVisualItem = paramVisualTable.getItem(i);
          localVisualItem.setVisible(true);
          localVisualItem.setEndVisible(true);
        }
        else
        {
          localVisualItem = paramVisualTable.addItem();
          localVisualItem.set("_label", this.m_nf.format(d8));
          localVisualItem.setDouble("_value", d8);
          double d9 = d3 == 0.0D ? 0.0D : (d8 - this.m_prevlo) / d3;
          if ((d9 <= 0.0D) || (d9 >= 1.0D)) {
            localVisualItem.setStartVisible(true);
          }
          set(localVisualItem, d9 * d1, localRectangle2D);
          set(localVisualItem, d7, localRectangle2D);
        }
      }
    }
  }
  
  protected void sqrtLayout(VisualTable paramVisualTable)
  {
    Rectangle2D localRectangle2D = getLayoutBounds();
    double d1 = getBreadth(localRectangle2D);
    double d2 = this.m_hi - this.m_lo;
    double d3 = MathLib.safeSqrt(this.m_prevlo);
    double d4 = MathLib.safeSqrt(this.m_prevhi) - d3;
    double d5 = Math.pow(10.0D, Math.floor(MathLib.safeLog10(this.m_lo)));
    double d6 = MathLib.safeSqrt(this.m_lo);
    double d7 = MathLib.safeSqrt(this.m_hi) - d6;
    Iterator localIterator = paramVisualTable.tuples();
    while (localIterator.hasNext())
    {
      localObject = (VisualItem)localIterator.next();
      reset((VisualItem)localObject);
      d8 = ((VisualItem)localObject).getDouble("_value");
      double d9 = d2 == 0.0D ? 0.0D : (MathLib.safeSqrt(d8) - d6) / d7 * d1;
      set((VisualItem)localObject, d9, localRectangle2D);
    }
    Object localObject = paramVisualTable.index("_value");
    double d8 = getLinearStep(d2, d1 / d2);
    if (d8 == 0.0D) {
      d8 = 1.0D;
    }
    for (double d11 = d5; d11 <= this.m_hi; d11 += d8)
    {
      double d10 = (MathLib.safeSqrt(d11) - d6) / d7 * d1;
      if (d10 >= -0.5D)
      {
        int i;
        VisualItem localVisualItem;
        if ((i = ((Index)localObject).get(d11)) >= 0)
        {
          localVisualItem = paramVisualTable.getItem(i);
          localVisualItem.setVisible(true);
          localVisualItem.setEndVisible(true);
        }
        else
        {
          localVisualItem = paramVisualTable.addItem();
          localVisualItem.set("_label", this.m_nf.format(d11));
          localVisualItem.setDouble("_value", d11);
          double d12 = d4 == 0.0D ? 0.0D : (MathLib.safeSqrt(d11) - d3) / d4;
          if ((d12 <= 0.0D) || (d12 >= 1.0D)) {
            localVisualItem.setStartVisible(true);
          }
          set(localVisualItem, d12 * d1, localRectangle2D);
          set(localVisualItem, d10, localRectangle2D);
        }
      }
    }
  }
  
  protected void logLayout(VisualTable paramVisualTable)
  {
    Rectangle2D localRectangle2D = getLayoutBounds();
    double d1 = getBreadth(localRectangle2D);
    paramVisualTable.clear();
    double d2 = MathLib.safeLog10(this.m_lo);
    double d3 = MathLib.safeLog10(this.m_hi);
    double d4 = d3 - d2;
    double d5 = MathLib.log10(d3 - d2);
    int i = (int)Math.floor(d5);
    int j = (int)Math.floor(d2);
    int k = (int)Math.ceil(d3);
    double d6 = Math.pow(10.0D, j);
    double d7 = Math.pow(10.0D, k);
    double d8 = d6 * Math.pow(10.0D, i);
    double d10 = d6;
    for (double d11 = 0.0D; d10 <= d7; d11 += 1.0D)
    {
      double d9 = MathLib.safeLog10(d10);
      if ((d11 != 0.0D) && (Math.abs(d9 - Math.round(d9)) < 1.0E-4D))
      {
        d11 = 0.0D;
        d8 = 10.0D * d8;
      }
      d9 = (d9 - d2) / d4 * d1;
      if (d9 >= -0.5D)
      {
        VisualItem localVisualItem = paramVisualTable.addItem();
        set(localVisualItem, d9, localRectangle2D);
        Object localObject = d11 == 0.0D ? this.m_nf.format(d10) : null;
        localVisualItem.set("_label", localObject);
        localVisualItem.setDouble("_value", d10);
      }
      d10 += d8;
    }
  }
  
  protected double getBreadth(Rectangle2D paramRectangle2D)
  {
    switch (this.m_axis)
    {
    case 0: 
      return paramRectangle2D.getWidth();
    }
    return paramRectangle2D.getHeight();
  }
  
  protected double adjust(double paramDouble)
  {
    switch (this.m_scale)
    {
    case 1: 
      return Math.pow(10.0D, paramDouble);
    case 2: 
      return paramDouble * paramDouble;
    }
    return paramDouble;
  }
  
  protected double getLinearStep(double paramDouble1, double paramDouble2)
  {
    double d1 = Math.log(paramDouble1) / Math.log(10.0D);
    double d2 = Math.pow(10.0D, Math.floor(d1));
    double d3 = d2 * paramDouble2;
    if (d3 > 1000.0D) {
      d2 /= 20.0D;
    } else if (d3 > 500.0D) {
      d2 /= 10.0D;
    } else if (d3 > 250.0D) {
      d2 /= 5.0D;
    } else if (d3 > 200.0D) {
      d2 /= 4.0D;
    } else if (d3 > 100.0D) {
      d2 /= 2.0D;
    }
    return d2;
  }
  
  protected void ordinalLayout(VisualTable paramVisualTable)
  {
    ObjectRangeModel localObjectRangeModel = (ObjectRangeModel)this.m_model;
    double d1 = this.m_hi - this.m_lo;
    double d2 = this.m_prevhi - this.m_prevlo;
    Rectangle2D localRectangle2D = getLayoutBounds();
    double d3 = getBreadth(localRectangle2D);
    double d4 = d3 / d1;
    int i = getOrdinalStep(d1, d4);
    if (i <= 0) {
      i = 1;
    }
    Iterator localIterator = paramVisualTable.tuples();
    while (localIterator.hasNext())
    {
      localObject = (VisualItem)localIterator.next();
      reset((VisualItem)localObject);
      double d5 = ((VisualItem)localObject).getDouble("_value");
      double d6 = d1 == 0.0D ? 0.0D : (d5 - this.m_lo) / d1 * d3;
      set((VisualItem)localObject, d6, localRectangle2D);
    }
    Object localObject = paramVisualTable.index("_value");
    int k = (int)this.m_lo;
    while (k <= this.m_hi)
    {
      int j;
      VisualItem localVisualItem;
      if ((j = ((Index)localObject).get(k)) >= 0)
      {
        localVisualItem = paramVisualTable.getItem(j);
        localVisualItem.set(VisualItem.LABEL, localObjectRangeModel.getObject(k).toString());
        localVisualItem.setVisible(true);
        localVisualItem.setEndVisible(true);
      }
      else
      {
        localVisualItem = paramVisualTable.addItem();
        localVisualItem.set(VisualItem.LABEL, localObjectRangeModel.getObject(k).toString());
        localVisualItem.setDouble(VisualItem.VALUE, k);
        double d7 = d2 == 0.0D ? 0.0D : (k - this.m_prevlo) / d2;
        if ((d7 <= 0.0D) || (d7 >= 1.0D)) {
          localVisualItem.setStartVisible(true);
        }
        set(localVisualItem, d7 * d3, localRectangle2D);
        set(localVisualItem, (k - this.m_lo) * d3 / d1, localRectangle2D);
      }
      k += i;
    }
  }
  
  protected int getOrdinalStep(double paramDouble1, double paramDouble2)
  {
    return paramDouble2 >= this.m_spacing ? 1 : (int)Math.ceil(this.m_spacing / paramDouble2);
  }
  
  protected void set(VisualItem paramVisualItem, double paramDouble, Rectangle2D paramRectangle2D)
  {
    switch (this.m_axis)
    {
    case 0: 
      paramDouble += paramRectangle2D.getMinX();
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.X, paramDouble);
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.Y, paramRectangle2D.getMinY());
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.X2, paramDouble);
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.Y2, paramRectangle2D.getMaxY());
      break;
    case 1: 
      paramDouble = paramRectangle2D.getMaxY() - paramDouble - 1.0D;
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.X, paramRectangle2D.getMinX());
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.Y, paramDouble);
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.X2, paramRectangle2D.getMaxX());
      PrefuseLib.updateDouble(paramVisualItem, VisualItem.Y2, paramDouble);
    }
  }
  
  protected void reset(VisualItem paramVisualItem)
  {
    paramVisualItem.setVisible(false);
    paramVisualItem.setEndVisible(false);
    paramVisualItem.setStartStrokeColor(paramVisualItem.getStrokeColor());
    paramVisualItem.revertToDefault(VisualItem.STROKECOLOR);
    paramVisualItem.revertToDefault(VisualItem.ENDSTROKECOLOR);
    paramVisualItem.setStartTextColor(paramVisualItem.getTextColor());
    paramVisualItem.revertToDefault(VisualItem.TEXTCOLOR);
    paramVisualItem.revertToDefault(VisualItem.ENDTEXTCOLOR);
    paramVisualItem.setStartFillColor(paramVisualItem.getFillColor());
    paramVisualItem.revertToDefault(VisualItem.FILLCOLOR);
    paramVisualItem.revertToDefault(VisualItem.ENDFILLCOLOR);
  }
  
  protected void garbageCollect(VisualTable paramVisualTable)
  {
    Iterator localIterator = paramVisualTable.tuples();
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      if ((!localVisualItem.isStartVisible()) && (!localVisualItem.isEndVisible())) {
        paramVisualTable.removeTuple(localVisualItem);
      }
    }
  }
  
  protected VisualTable getTable()
  {
    TupleSet localTupleSet = this.m_vis.getGroup(this.m_group);
    if (localTupleSet == null)
    {
      Schema localSchema = PrefuseLib.getAxisLabelSchema();
      VisualTable localVisualTable = this.m_vis.addTable(this.m_group, localSchema);
      localVisualTable.index("_value");
      return localVisualTable;
    }
    if ((localTupleSet instanceof VisualTable)) {
      return (VisualTable)localTupleSet;
    }
    throw new IllegalStateException("Group already exists, not being used for labels");
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\AxisLabelLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */