package prefuse.data.column;

import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import prefuse.data.Table;
import prefuse.util.ArrayLib;
import prefuse.util.TypeLib;
import prefuse.util.collections.DefaultLiteralComparator;
import prefuse.util.collections.IntIterator;

public class Columns
{
  public static Object[] toArray(Table paramTable, String paramString)
  {
    int i = paramTable.getColumnNumber(paramString);
    IntIterator localIntIterator = paramTable.rows();
    Object[] arrayOfObject = new Object[paramTable.getRowCount()];
    for (int j = 0; localIntIterator.hasNext(); j++) {
      arrayOfObject[j] = paramTable.getValueAt(localIntIterator.nextInt(), i);
    }
    return arrayOfObject;
  }
  
  public static double[] toDoubleArray(Table paramTable, String paramString)
  {
    if (!paramTable.canGetDouble(paramString)) {
      throw new IllegalArgumentException("The table column must support returning values of type double");
    }
    IntIterator localIntIterator = paramTable.rows();
    double[] arrayOfDouble = new double[paramTable.getRowCount()];
    for (int i = 0; localIntIterator.hasNext(); i++) {
      arrayOfDouble[i] = paramTable.getDouble(localIntIterator.nextInt(), paramString);
    }
    return arrayOfDouble;
  }
  
  public static Object[] ordinalArray(Table paramTable, String paramString)
  {
    return ordinalArray(paramTable, paramString, DefaultLiteralComparator.getInstance());
  }
  
  public static Object[] ordinalArray(Table paramTable, String paramString, Comparator paramComparator)
  {
    int i = paramTable.getColumnNumber(paramString);
    IntIterator localIntIterator = paramTable.rows();
    HashSet localHashSet = new HashSet(Math.min(10, paramTable.getRowCount() / 2));
    while (localIntIterator.hasNext()) {
      localHashSet.add(paramTable.getValueAt(localIntIterator.nextInt(), i));
    }
    Object[] arrayOfObject = localHashSet.toArray();
    Arrays.sort(arrayOfObject, paramComparator);
    return arrayOfObject;
  }
  
  public static Map ordinalMap(Table paramTable, String paramString)
  {
    return ordinalMap(paramTable, paramString, DefaultLiteralComparator.getInstance());
  }
  
  public static Map ordinalMap(Table paramTable, String paramString, Comparator paramComparator)
  {
    Object[] arrayOfObject = ordinalArray(paramTable, paramString, paramComparator);
    HashMap localHashMap = new HashMap();
    for (int i = 0; i < arrayOfObject.length; i++) {
      localHashMap.put(arrayOfObject[i], new Integer(i));
    }
    return localHashMap;
  }
  
  public static int uniqueCount(Table paramTable, String paramString)
  {
    int i = paramTable.getColumnNumber(paramString);
    IntIterator localIntIterator = paramTable.rows();
    HashSet localHashSet = new HashSet(Math.min(10, paramTable.getRowCount() / 2));
    while (localIntIterator.hasNext()) {
      localHashSet.add(paramTable.getValueAt(localIntIterator.nextInt(), i));
    }
    return localHashSet.size();
  }
  
  public static int min(Table paramTable, String paramString)
  {
    return min(paramTable, paramString, DefaultLiteralComparator.getInstance());
  }
  
  public static int min(Table paramTable, String paramString, Comparator paramComparator)
  {
    IntIterator localIntIterator = paramTable.rows();
    int j = -1;
    int k = paramTable.getColumnNumber(paramString);
    Object localObject1 = null;
    if (localIntIterator.hasNext())
    {
      j = localIntIterator.nextInt();
      localObject1 = paramTable.get(j, paramString);
    }
    while (localIntIterator.hasNext())
    {
      int i = localIntIterator.nextInt();
      Object localObject2 = paramTable.getValueAt(i, k);
      if (paramComparator.compare(localObject2, localObject1) < 0)
      {
        j = i;
        localObject1 = localObject2;
      }
    }
    return j;
  }
  
  public static int max(Table paramTable, String paramString)
  {
    return max(paramTable, paramString, DefaultLiteralComparator.getInstance());
  }
  
  public static int max(Table paramTable, String paramString, Comparator paramComparator)
  {
    IntIterator localIntIterator = paramTable.rows();
    int j = -1;
    int k = paramTable.getColumnNumber(paramString);
    Object localObject1 = null;
    if (localIntIterator.hasNext())
    {
      j = localIntIterator.nextInt();
      localObject1 = paramTable.get(j, paramString);
    }
    while (localIntIterator.hasNext())
    {
      int i = localIntIterator.nextInt();
      Object localObject2 = paramTable.getValueAt(i, k);
      if (paramComparator.compare(localObject2, localObject1) > 0)
      {
        j = i;
        localObject1 = localObject2;
      }
    }
    return j;
  }
  
  public static int median(Table paramTable, String paramString)
  {
    return median(paramTable, paramString, DefaultLiteralComparator.getInstance());
  }
  
  public static int median(Table paramTable, String paramString, Comparator paramComparator)
  {
    int i = paramTable.getColumnNumber(paramString);
    IntIterator localIntIterator = paramTable.rows();
    int[] arrayOfInt = new int[paramTable.getRowCount()];
    Object[] arrayOfObject = new Object[paramTable.getRowCount()];
    for (int j = 0; localIntIterator.hasNext(); j++)
    {
      arrayOfInt[j] = localIntIterator.nextInt();
      arrayOfObject[j] = paramTable.getValueAt(arrayOfInt[j], i);
    }
    ArrayLib.sort(arrayOfObject, arrayOfInt, paramComparator);
    return arrayOfInt[(arrayOfInt.length / 2)];
  }
  
  public static double mean(Table paramTable, String paramString)
  {
    int i = paramTable.getColumnNumber(paramString);
    Column localColumn = paramTable.getColumn(i);
    if (TypeLib.isNumericType(localColumn.getColumnClass()))
    {
      IntIterator localIntIterator = paramTable.rows();
      int k = 0;
      double d = 0.0D;
      while (localIntIterator.hasNext())
      {
        int j = paramTable.getColumnRow(localIntIterator.nextInt(), i);
        d += localColumn.getDouble(j);
        k++;
      }
      return d / k;
    }
    return NaN.0D;
  }
  
  public static double deviation(Table paramTable, String paramString)
  {
    return deviation(paramTable, paramString, mean(paramTable, paramString));
  }
  
  public static double deviation(Table paramTable, String paramString, double paramDouble)
  {
    int i = paramTable.getColumnNumber(paramString);
    Column localColumn = paramTable.getColumn(i);
    if (TypeLib.isNumericType(localColumn.getColumnClass()))
    {
      IntIterator localIntIterator = paramTable.rows();
      int k = 0;
      double d1 = 0.0D;
      while (localIntIterator.hasNext())
      {
        int j = paramTable.getColumnRow(localIntIterator.nextInt(), i);
        double d2 = localColumn.getDouble(j) - paramDouble;
        d1 += d2 * d2;
        k++;
      }
      return Math.sqrt(d1 / k);
    }
    return NaN.0D;
  }
  
  public static double sum(Table paramTable, String paramString)
  {
    int i = paramTable.getColumnNumber(paramString);
    Column localColumn = paramTable.getColumn(i);
    if (TypeLib.isNumericType(localColumn.getColumnClass()))
    {
      IntIterator localIntIterator = paramTable.rows();
      int j;
      for (double d = 0.0D; localIntIterator.hasNext(); d += localColumn.getDouble(j)) {
        j = paramTable.getColumnRow(localIntIterator.nextInt(), i);
      }
      return d;
    }
    return NaN.0D;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\column\Columns.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */