package prefuse.data.expression;

import java.util.HashMap;
import prefuse.visual.expression.GroupSizeFunction;
import prefuse.visual.expression.HoverPredicate;
import prefuse.visual.expression.InGroupPredicate;
import prefuse.visual.expression.QueryExpression;
import prefuse.visual.expression.SearchPredicate;
import prefuse.visual.expression.ValidatedPredicate;
import prefuse.visual.expression.VisiblePredicate;

public class FunctionTable
{
  private static HashMap s_functionTable = new HashMap();
  
  public static boolean hasFunction(String paramString)
  {
    return s_functionTable.containsKey(paramString);
  }
  
  public static void addFunction(String paramString, Class paramClass)
  {
    if (!Function.class.isAssignableFrom(paramClass)) {
      throw new IllegalArgumentException("Type argument must be a subclass of FunctionExpression.");
    }
    if (hasFunction(paramString)) {
      throw new IllegalArgumentException("Function with that name already exists");
    }
    String str1 = paramString.toLowerCase();
    String str2 = paramString.toUpperCase();
    if ((!paramString.equals(str1)) && (!paramString.equals(str2))) {
      throw new IllegalArgumentException("Name can't have mixed case, try \"" + str2 + "\" instead.");
    }
    s_functionTable.put(str1, paramClass);
    s_functionTable.put(str2, paramClass);
  }
  
  public static Function createFunction(String paramString)
  {
    Class localClass = (Class)s_functionTable.get(paramString);
    if (localClass == null) {
      throw new IllegalArgumentException("Unrecognized function name");
    }
    try
    {
      return (Function)localClass.newInstance();
    }
    catch (InstantiationException localInstantiationException)
    {
      throw new RuntimeException(localInstantiationException);
    }
    catch (IllegalAccessException localIllegalAccessException)
    {
      throw new RuntimeException(localIllegalAccessException);
    }
  }
  
  static
  {
    addFunction("ABS", AbsFunction.class);
    addFunction("ACOS", AcosFunction.class);
    addFunction("ASIN", AsinFunction.class);
    addFunction("ATAN", AtanFunction.class);
    addFunction("ATAN2", Atan2Function.class);
    addFunction("CEIL", CeilFunction.class);
    addFunction("CEILING", CeilFunction.class);
    addFunction("COS", CosFunction.class);
    addFunction("COT", CotFunction.class);
    addFunction("DEGREES", DegreesFunction.class);
    addFunction("E", EFunction.class);
    addFunction("EXP", ExpFunction.class);
    addFunction("FLOOR", FloorFunction.class);
    addFunction("LOG", LogFunction.class);
    addFunction("LOG2", Log2Function.class);
    addFunction("LOG10", Log10Function.class);
    addFunction("MAX", MaxFunction.class);
    addFunction("MIN", MaxFunction.class);
    addFunction("MOD", MaxFunction.class);
    addFunction("PI", PiFunction.class);
    addFunction("POW", PowFunction.class);
    addFunction("POWER", PowFunction.class);
    addFunction("RADIANS", RadiansFunction.class);
    addFunction("RAND", RandFunction.class);
    addFunction("ROUND", RoundFunction.class);
    addFunction("SIGN", SignFunction.class);
    addFunction("SIN", SinFunction.class);
    addFunction("SQRT", SqrtFunction.class);
    addFunction("SUM", SumFunction.class);
    addFunction("TAN", TanFunction.class);
    addFunction("SAFELOG10", SafeLog10Function.class);
    addFunction("SAFESQRT", SafeSqrtFunction.class);
    addFunction("CAP", CapFunction.class);
    addFunction("CONCAT", ConcatFunction.class);
    addFunction("CONCAT_WS", ConcatWsFunction.class);
    addFunction("FORMAT", FormatFunction.class);
    addFunction("INSERT", RPadFunction.class);
    addFunction("LENGTH", LengthFunction.class);
    addFunction("LOWER", LowerFunction.class);
    addFunction("LCASE", LowerFunction.class);
    addFunction("LEFT", LeftFunction.class);
    addFunction("LPAD", LPadFunction.class);
    addFunction("MID", SubstringFunction.class);
    addFunction("POSITION", PositionFunction.class);
    addFunction("REVERSE", ReverseFunction.class);
    addFunction("REPEAT", RepeatFunction.class);
    addFunction("REPLACE", ReplaceFunction.class);
    addFunction("RIGHT", RightFunction.class);
    addFunction("RPAD", RPadFunction.class);
    addFunction("SPACE", SpaceFunction.class);
    addFunction("SUBSTRING", SubstringFunction.class);
    addFunction("UPPER", UpperFunction.class);
    addFunction("UCASE", UpperFunction.class);
    addFunction("RGB", RGBFunction.class);
    addFunction("RGBA", RGBAFunction.class);
    addFunction("GRAY", GrayFunction.class);
    addFunction("HEX", HexFunction.class);
    addFunction("HSB", HSBFunction.class);
    addFunction("HSBA", HSBAFunction.class);
    addFunction("COLORINTERP", ColorInterpFunction.class);
    addFunction("GROUPSIZE", GroupSizeFunction.class);
    addFunction("HOVER", HoverPredicate.class);
    addFunction("INGROUP", InGroupPredicate.class);
    addFunction("MATCH", SearchPredicate.class);
    addFunction("QUERY", QueryExpression.class);
    addFunction("VISIBLE", VisiblePredicate.class);
    addFunction("VALIDATED", ValidatedPredicate.class);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\expression\FunctionTable.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */