package prefuse.data.io;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import prefuse.data.Table;
import prefuse.data.parser.DataParseException;
import prefuse.data.parser.DataParser;
import prefuse.data.parser.ParserFactory;
import prefuse.data.parser.TypeInferencer;
import prefuse.util.collections.ByteArrayList;
import prefuse.util.io.IOLib;

public abstract class AbstractTextTableReader
  extends AbstractTableReader
{
  private ParserFactory m_pfactory;
  private boolean m_hasHeader;
  
  public AbstractTextTableReader()
  {
    this(ParserFactory.getDefaultFactory());
  }
  
  public AbstractTextTableReader(ParserFactory paramParserFactory)
  {
    this.m_pfactory = paramParserFactory;
    this.m_hasHeader = true;
  }
  
  public void setHasHeader(boolean paramBoolean)
  {
    this.m_hasHeader = paramBoolean;
  }
  
  public Table readTable(InputStream paramInputStream)
    throws DataIOException
  {
    ByteArrayList localByteArrayList = null;
    if (paramInputStream.markSupported())
    {
      paramInputStream.mark(Integer.MAX_VALUE);
    }
    else
    {
      try
      {
        localByteArrayList = IOLib.readAsBytes(paramInputStream);
      }
      catch (IOException localIOException1)
      {
        throw new DataIOException(localIOException1);
      }
      paramInputStream = localByteArrayList.getAsInputStream();
    }
    final TypeInferencer localTypeInferencer = new TypeInferencer(this.m_pfactory);
    final ArrayList localArrayList = new ArrayList();
    final int[] arrayOfInt = { 0, 0 };
    TableReadListener local1 = new TableReadListener()
    {
      int prevLine = -1;
      
      public void readValue(int paramAnonymousInt1, int paramAnonymousInt2, String paramAnonymousString)
        throws DataParseException
      {
        if ((paramAnonymousInt1 > 1) || (!AbstractTextTableReader.this.m_hasHeader))
        {
          localTypeInferencer.sample(paramAnonymousInt2 - 1, paramAnonymousString);
          if (paramAnonymousInt1 != this.prevLine)
          {
            this.prevLine = paramAnonymousInt1;
            arrayOfInt[0] += 1;
          }
        }
        else if ((paramAnonymousInt1 == 1) && (AbstractTextTableReader.this.m_hasHeader))
        {
          localArrayList.add(paramAnonymousString);
        }
        if (paramAnonymousInt2 > arrayOfInt[1]) {
          arrayOfInt[1] = paramAnonymousInt2;
        }
      }
    };
    try
    {
      read(paramInputStream, local1);
    }
    catch (IOException localIOException2)
    {
      throw new DataIOException(localIOException2);
    }
    catch (DataParseException localDataParseException1) {}
    int i = arrayOfInt[0];
    int j = arrayOfInt[1];
    final Table localTable = new Table(i, j);
    for (int k = 0; k < j; k++)
    {
      String str;
      if (this.m_hasHeader) {
        str = (String)localArrayList.get(k);
      } else {
        str = getDefaultHeader(k);
      }
      localTable.addColumn(str, localTypeInferencer.getType(k));
    }
    arrayOfInt[0] = (arrayOfInt[1] = -1);
    TableReadListener local2 = new TableReadListener()
    {
      int prevLine = -1;
      
      public void readValue(int paramAnonymousInt1, int paramAnonymousInt2, String paramAnonymousString)
        throws DataParseException
      {
        if ((paramAnonymousInt1 == 1) && (AbstractTextTableReader.this.m_hasHeader)) {
          return;
        }
        if (paramAnonymousInt1 != this.prevLine)
        {
          this.prevLine = paramAnonymousInt1;
          arrayOfInt[0] += 1;
        }
        arrayOfInt[1] = (paramAnonymousInt2 - 1);
        DataParser localDataParser = localTypeInferencer.getParser(arrayOfInt[1]);
        localTable.setValueAt(arrayOfInt[0], arrayOfInt[1], localDataParser.parse(paramAnonymousString));
      }
    };
    try
    {
      if (paramInputStream.markSupported()) {
        paramInputStream.reset();
      } else {
        paramInputStream = localByteArrayList.getAsInputStream();
      }
      read(paramInputStream, local2);
    }
    catch (IOException localIOException3)
    {
      throw new DataIOException(localIOException3);
    }
    catch (DataParseException localDataParseException2)
    {
      throw new DataIOException("Parse exception for column \"" + arrayOfInt[1] + '"' + " at row: " + arrayOfInt[0], localDataParseException2);
    }
    return localTable;
  }
  
  public static String getDefaultHeader(int paramInt)
  {
    if (paramInt == 0) {
      return "A";
    }
    int i = (int)(Math.log(paramInt) / Math.log(26.0D)) + 1;
    char[] arrayOfChar = new char[i];
    int j = i;
    arrayOfChar[(--j)] = ((char)(65 + paramInt % 26));
    paramInt /= 26;
    while (paramInt > 26)
    {
      arrayOfChar[(--j)] = ((char)(65 + paramInt % 26));
      paramInt /= 26;
    }
    if (paramInt > 0) {
      arrayOfChar[(--j)] = ((char)(65 + (paramInt - 1) % 26));
    }
    return new String(arrayOfChar, j, i);
  }
  
  protected abstract void read(InputStream paramInputStream, TableReadListener paramTableReadListener)
    throws IOException, DataParseException;
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\AbstractTextTableReader.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */