package prefuse.data.io;

import java.io.InputStream;
import java.io.PrintStream;
import java.util.Date;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import org.xml.sax.Attributes;
import org.xml.sax.helpers.DefaultHandler;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Table;
import prefuse.data.Tree;
import prefuse.data.parser.DataParseException;
import prefuse.data.parser.DataParser;
import prefuse.data.parser.ParserFactory;

public class TreeMLReader
  extends AbstractGraphReader
{
  private ParserFactory m_pf = ParserFactory.getDefaultFactory();
  
  public Graph readGraph(InputStream paramInputStream)
    throws DataIOException
  {
    try
    {
      TreeMLHandler localTreeMLHandler = new TreeMLHandler();
      SAXParserFactory localSAXParserFactory = SAXParserFactory.newInstance();
      SAXParser localSAXParser = localSAXParserFactory.newSAXParser();
      localSAXParser.parse(paramInputStream, localTreeMLHandler);
      return localTreeMLHandler.getTree();
    }
    catch (Exception localException)
    {
      throw new DataIOException(localException);
    }
  }
  
  public class TreeMLHandler
    extends DefaultHandler
    implements TreeMLReader.Tokens
  {
    private Table m_nodes = null;
    private Tree m_tree = null;
    private Node m_activeNode = null;
    private boolean m_inSchema = true;
    
    public TreeMLHandler() {}
    
    public void startDocument()
    {
      this.m_tree = new Tree();
      this.m_nodes = this.m_tree.getNodeTable();
    }
    
    private void schemaCheck()
    {
      if (this.m_inSchema) {
        this.m_inSchema = false;
      }
    }
    
    public void endElement(String paramString1, String paramString2, String paramString3)
    {
      if ((paramString3.equals("branch")) || (paramString3.equals("leaf"))) {
        this.m_activeNode = this.m_activeNode.getParent();
      }
    }
    
    public void startElement(String paramString1, String paramString2, String paramString3, Attributes paramAttributes)
    {
      Object localObject;
      if (paramString3.equals("attributeDecl"))
      {
        if (!this.m_inSchema) {
          throw new RuntimeException("All declarations must be done before nodes begin");
        }
        localObject = paramAttributes.getValue("name");
        String str = paramAttributes.getValue("type");
        Class localClass = parseType(str);
        this.m_nodes.addColumn((String)localObject, localClass);
      }
      else if ((paramString3.equals("branch")) || (paramString3.equals("leaf")))
      {
        schemaCheck();
        if (this.m_activeNode == null) {
          localObject = this.m_tree.addRoot();
        } else {
          localObject = this.m_tree.addChild(this.m_activeNode);
        }
        this.m_activeNode = ((Node)localObject);
      }
      else if (paramString3.equals("attribute"))
      {
        parseAttribute(paramAttributes);
      }
    }
    
    protected void parseAttribute(Attributes paramAttributes)
    {
      String str2 = null;
      String str3 = null;
      for (int i = 0; i < paramAttributes.getLength(); i++)
      {
        String str1 = paramAttributes.getQName(i);
        if (str1.equals("name")) {
          str2 = paramAttributes.getValue(i);
        } else if (str1.equals("value")) {
          str3 = paramAttributes.getValue(i);
        }
      }
      if ((str2 == null) || (str3 == null))
      {
        System.err.println("Attribute under-specified");
        return;
      }
      try
      {
        Object localObject = parse(str3, this.m_nodes.getColumnType(str2));
        this.m_activeNode.set(str2, localObject);
      }
      catch (Exception localException)
      {
        throw new RuntimeException(localException);
      }
    }
    
    protected Object parse(String paramString, Class paramClass)
      throws DataParseException
    {
      DataParser localDataParser = TreeMLReader.this.m_pf.getParser(paramClass);
      return localDataParser.parse(paramString);
    }
    
    protected Class parseType(String paramString)
    {
      paramString = Character.toUpperCase(paramString.charAt(0)) + paramString.substring(1).toLowerCase();
      if ((paramString.equals("Int")) || (paramString.equals("Integer"))) {
        return Integer.TYPE;
      }
      if (paramString.equals("Long")) {
        return Long.TYPE;
      }
      if (paramString.equals("Float")) {
        return Float.TYPE;
      }
      if ((paramString.equals("Double")) || (paramString.equals("Real"))) {
        return Double.TYPE;
      }
      if (paramString.equals("Boolean")) {
        return Boolean.TYPE;
      }
      if (paramString.equals("String")) {
        return String.class;
      }
      if (paramString.equals("Date")) {
        return Date.class;
      }
      throw new RuntimeException("Unrecognized data type: " + paramString);
    }
    
    public Tree getTree()
    {
      return this.m_tree;
    }
  }
  
  public static abstract interface Tokens
  {
    public static final String TREE = "tree";
    public static final String BRANCH = "branch";
    public static final String LEAF = "leaf";
    public static final String ATTR = "attribute";
    public static final String NAME = "name";
    public static final String VALUE = "value";
    public static final String TYPE = "type";
    public static final String DECLS = "declarations";
    public static final String DECL = "attributeDecl";
    public static final String INT = "Int";
    public static final String INTEGER = "Integer";
    public static final String LONG = "Long";
    public static final String FLOAT = "Float";
    public static final String REAL = "Real";
    public static final String STRING = "String";
    public static final String DATE = "Date";
    public static final String CATEGORY = "Category";
    public static final String BOOLEAN = "Boolean";
    public static final String DOUBLE = "Double";
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\TreeMLReader.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */