package prefuse.data.io;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.Date;
import java.util.HashMap;
import prefuse.data.Graph;
import prefuse.data.Node;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tree;
import prefuse.util.io.XMLWriter;

public class TreeMLWriter
  extends AbstractGraphWriter
{
  private static final HashMap TYPES = new HashMap();
  
  public void writeGraph(Graph paramGraph, OutputStream paramOutputStream)
    throws DataIOException
  {
    Schema localSchema = paramGraph.getNodeTable().getSchema();
    checkTreeMLSchema(localSchema);
    XMLWriter localXMLWriter = new XMLWriter(new PrintWriter(paramOutputStream));
    localXMLWriter.begin();
    localXMLWriter.comment("prefuse TreeML Writer | " + new Date(System.currentTimeMillis()));
    localXMLWriter.start("tree");
    localXMLWriter.start("declarations");
    String[] arrayOfString1 = { "name", "type" };
    String[] arrayOfString2 = new String[2];
    for (int i = 0; i < localSchema.getColumnCount(); i++)
    {
      arrayOfString2[0] = localSchema.getColumnName(i);
      arrayOfString2[1] = ((String)TYPES.get(localSchema.getColumnType(i)));
      localXMLWriter.tag("attributeDecl", arrayOfString1, arrayOfString2, 2);
    }
    localXMLWriter.end();
    localXMLWriter.println();
    arrayOfString1[0] = "name";
    arrayOfString1[1] = "value";
    for (Node localNode = paramGraph.getSpanningTree().getRoot(); localNode != null; localNode = nextNode(localNode, localXMLWriter))
    {
      int j = localNode.getChildCount() == 0 ? 1 : 0;
      if (j != 0) {
        localXMLWriter.start("leaf");
      } else {
        localXMLWriter.start("branch");
      }
      if (localSchema.getColumnCount() > 0) {
        for (int k = 0; k < localSchema.getColumnCount(); k++)
        {
          arrayOfString2[0] = localSchema.getColumnName(k);
          arrayOfString2[1] = localNode.getString(arrayOfString2[0]);
          localXMLWriter.tag("attribute", arrayOfString1, arrayOfString2, 2);
        }
      }
    }
    localXMLWriter.end();
    localXMLWriter.finish();
  }
  
  private Node nextNode(Node paramNode, XMLWriter paramXMLWriter)
  {
    Object localObject;
    if ((localObject = paramNode.getChild(0)) == null) {
      if ((localObject = paramNode.getNextSibling()) != null)
      {
        paramXMLWriter.end();
      }
      else
      {
        localObject = paramNode.getParent();
        paramXMLWriter.end();
        while (localObject != null)
        {
          Node localNode;
          if ((localNode = ((Node)localObject).getNextSibling()) != null)
          {
            localObject = localNode;
            paramXMLWriter.end();
            break;
          }
          localObject = ((Node)localObject).getParent();
          paramXMLWriter.end();
        }
      }
    }
    return (Node)localObject;
  }
  
  private void checkTreeMLSchema(Schema paramSchema)
    throws DataIOException
  {
    for (int i = 0; i < paramSchema.getColumnCount(); i++)
    {
      Class localClass = paramSchema.getColumnType(i);
      if (TYPES.get(localClass) == null) {
        throw new DataIOException("Data type unsupported by the TreeML format: " + localClass.getName());
      }
    }
  }
  
  static
  {
    TYPES.put(Integer.TYPE, "Int");
    TYPES.put(Long.TYPE, "Long");
    TYPES.put(Float.TYPE, "Float");
    TYPES.put(Double.TYPE, "Real");
    TYPES.put(Boolean.TYPE, "Boolean");
    TYPES.put(String.class, "String");
    TYPES.put(Date.class, "Date");
  }
  
  public static abstract interface Tokens
    extends TreeMLReader.Tokens
  {}
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\io\TreeMLWriter.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */