package prefuse.data.search;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import prefuse.data.Tuple;

public class Trie
{
  private TrieBranch root = new TrieBranch();
  private boolean caseSensitive = false;
  
  public Trie(boolean paramBoolean)
  {
    this.caseSensitive = paramBoolean;
  }
  
  public void addString(String paramString, Tuple paramTuple)
  {
    TrieLeaf localTrieLeaf = new TrieLeaf(paramString, paramTuple);
    addLeaf(this.root, localTrieLeaf, 0);
  }
  
  public void removeString(String paramString, Tuple paramTuple)
  {
    removeLeaf(this.root, paramString, paramTuple, 0);
  }
  
  private final int getIndex(char[] paramArrayOfChar, char paramChar)
  {
    for (int i = 0; i < paramArrayOfChar.length; i++) {
      if (paramArrayOfChar[i] == paramChar) {
        return i;
      }
    }
    return -1;
  }
  
  private final char getChar(String paramString, int paramInt)
  {
    char c = (paramInt < 0) || (paramInt >= paramString.length()) ? '\000' : paramString.charAt(paramInt);
    return this.caseSensitive ? c : Character.toLowerCase(c);
  }
  
  private final TrieNode equalityCheck(String paramString, TrieLeaf paramTrieLeaf)
  {
    if (this.caseSensitive) {
      return paramTrieLeaf.word.startsWith(paramString) ? paramTrieLeaf : null;
    }
    int i = paramString.length();
    if (i > paramTrieLeaf.word.length()) {
      return null;
    }
    for (int j = 0; j < i; j++)
    {
      int k = Character.toLowerCase(paramString.charAt(j));
      int m = Character.toLowerCase(paramTrieLeaf.word.charAt(j));
      if (k != m) {
        return null;
      }
    }
    return paramTrieLeaf;
  }
  
  private boolean removeLeaf(TrieBranch paramTrieBranch, String paramString, Tuple paramTuple, int paramInt)
  {
    char c = getChar(paramString, paramInt);
    int i = getIndex(paramTrieBranch.chars, c);
    if (i == -1) {
      return false;
    }
    TrieNode localTrieNode = paramTrieBranch.children[i];
    if ((localTrieNode instanceof TrieBranch))
    {
      localObject = (TrieBranch)localTrieNode;
      boolean bool = removeLeaf((TrieBranch)localObject, paramString, paramTuple, paramInt + 1);
      if (bool)
      {
        paramTrieBranch.leafCount -= 1;
        if (((TrieBranch)localObject).leafCount == 1) {
          paramTrieBranch.children[i] = localObject.children[1];
        }
      }
      return bool;
    }
    Object localObject = (TrieLeaf)localTrieNode;
    if (((TrieLeaf)localObject).tuple == paramTuple)
    {
      paramTrieBranch.children[i] = ((TrieLeaf)localObject).next;
      if (((TrieLeaf)localObject).next == null) {
        repairBranch(paramTrieBranch, i);
      }
      paramTrieBranch.leafCount -= 1;
      return true;
    }
    for (TrieLeaf localTrieLeaf1 = ((TrieLeaf)localObject).next; (localTrieLeaf1 != null) && (localTrieLeaf1.tuple != paramTuple); localTrieLeaf1 = localTrieLeaf1.next) {
      localObject = localTrieLeaf1;
    }
    if (localTrieLeaf1 == null) {
      return false;
    }
    for (TrieLeaf localTrieLeaf2 = (TrieLeaf)localTrieNode; localTrieLeaf2.tuple != paramTuple; localTrieLeaf2 = localTrieLeaf2.next) {
      localTrieLeaf2.leafCount -= 1;
    }
    ((TrieLeaf)localObject).next = localTrieLeaf1.next;
    paramTrieBranch.leafCount -= 1;
    return true;
  }
  
  private void repairBranch(TrieBranch paramTrieBranch, int paramInt)
  {
    if (paramInt == 0)
    {
      paramTrieBranch.children[0] = null;
    }
    else
    {
      int i = paramTrieBranch.chars.length;
      char[] arrayOfChar = new char[i - 1];
      TrieNode[] arrayOfTrieNode = new TrieNode[i - 1];
      System.arraycopy(paramTrieBranch.chars, 0, arrayOfChar, 0, paramInt);
      System.arraycopy(paramTrieBranch.children, 0, arrayOfTrieNode, 0, paramInt);
      System.arraycopy(paramTrieBranch.chars, paramInt + 1, arrayOfChar, paramInt, i - paramInt - 1);
      System.arraycopy(paramTrieBranch.children, paramInt + 1, arrayOfTrieNode, paramInt, i - paramInt - 1);
      paramTrieBranch.chars = arrayOfChar;
      paramTrieBranch.children = arrayOfTrieNode;
    }
  }
  
  private void addLeaf(TrieBranch paramTrieBranch, TrieLeaf paramTrieLeaf, int paramInt)
  {
    paramTrieBranch.leafCount += paramTrieLeaf.leafCount;
    char c = getChar(paramTrieLeaf.word, paramInt);
    int i = getIndex(paramTrieBranch.chars, c);
    if (i == -1)
    {
      addChild(paramTrieBranch, paramTrieLeaf, c);
    }
    else
    {
      TrieNode localTrieNode = paramTrieBranch.children[i];
      if (localTrieNode == null)
      {
        paramTrieBranch.children[i] = paramTrieLeaf;
      }
      else if ((localTrieNode instanceof TrieBranch))
      {
        addLeaf((TrieBranch)localTrieNode, paramTrieLeaf, paramInt + 1);
      }
      else
      {
        TrieLeaf localTrieLeaf = (TrieLeaf)localTrieNode;
        if ((i == 0) || (this.caseSensitive ? localTrieLeaf.word.equals(paramTrieLeaf.word) : localTrieLeaf.word.equalsIgnoreCase(paramTrieLeaf.word)))
        {
          while (localTrieLeaf.next != null)
          {
            localTrieLeaf.leafCount += 1;
            localTrieLeaf = localTrieLeaf.next;
          }
          localTrieLeaf.leafCount += 1;
          localTrieLeaf.next = paramTrieLeaf;
        }
        else
        {
          TrieBranch localTrieBranch = new TrieBranch();
          paramTrieBranch.children[i] = localTrieBranch;
          addLeaf(localTrieBranch, localTrieLeaf, paramInt + 1);
          addLeaf(localTrieBranch, paramTrieLeaf, paramInt + 1);
        }
      }
    }
  }
  
  private void addChild(TrieBranch paramTrieBranch, TrieNode paramTrieNode, char paramChar)
  {
    int i = paramTrieBranch.chars.length;
    char[] arrayOfChar = new char[i + 1];
    TrieNode[] arrayOfTrieNode = new TrieNode[i + 1];
    System.arraycopy(paramTrieBranch.chars, 0, arrayOfChar, 0, i);
    System.arraycopy(paramTrieBranch.children, 0, arrayOfTrieNode, 0, i);
    arrayOfChar[i] = paramChar;
    arrayOfTrieNode[i] = paramTrieNode;
    paramTrieBranch.chars = arrayOfChar;
    paramTrieBranch.children = arrayOfTrieNode;
  }
  
  public TrieNode find(String paramString)
  {
    return paramString.length() < 1 ? null : find(paramString, this.root, 0);
  }
  
  private TrieNode find(String paramString, TrieBranch paramTrieBranch, int paramInt)
  {
    char c = getChar(paramString, paramInt);
    int i = getIndex(paramTrieBranch.chars, c);
    if (i == -1) {
      return null;
    }
    if (paramString.length() - 1 == paramInt) {
      return paramTrieBranch.children[i];
    }
    if ((paramTrieBranch.children[i] instanceof TrieLeaf)) {
      return equalityCheck(paramString, (TrieLeaf)paramTrieBranch.children[i]);
    }
    return find(paramString, (TrieBranch)paramTrieBranch.children[i], paramInt + 1);
  }
  
  public class TrieIterator
    implements Iterator
  {
    private LinkedList queue = new LinkedList();
    
    public TrieIterator(Trie.TrieNode paramTrieNode)
    {
      this.queue.add(paramTrieNode);
    }
    
    public boolean hasNext()
    {
      return !this.queue.isEmpty();
    }
    
    public Object next()
    {
      if (this.queue.isEmpty()) {
        throw new NoSuchElementException();
      }
      Trie.TrieNode localTrieNode = (Trie.TrieNode)this.queue.removeFirst();
      if ((localTrieNode instanceof Trie.TrieLeaf))
      {
        localObject = (Trie.TrieLeaf)localTrieNode;
        Tuple localTuple = ((Trie.TrieLeaf)localObject).tuple;
        if (((Trie.TrieLeaf)localObject).next != null) {
          this.queue.addFirst(((Trie.TrieLeaf)localObject).next);
        }
        return localTuple;
      }
      Object localObject = (Trie.TrieBranch)localTrieNode;
      for (int i = ((Trie.TrieBranch)localObject).chars.length - 1; i > 0; i--) {
        this.queue.addFirst(localObject.children[i]);
      }
      if (localObject.children[0] != null) {
        this.queue.addFirst(localObject.children[0]);
      }
      return next();
    }
    
    public void remove()
    {
      throw new UnsupportedOperationException();
    }
  }
  
  public class TrieLeaf
    extends Trie.TrieNode
  {
    String word;
    Tuple tuple;
    TrieLeaf next;
    
    public TrieLeaf(String paramString, Tuple paramTuple)
    {
      super();
      this.word = paramString;
      this.tuple = paramTuple;
      this.next = null;
      this.leafCount = 1;
    }
  }
  
  public class TrieBranch
    extends Trie.TrieNode
  {
    char[] chars = { '\000' };
    Trie.TrieNode[] children = new Trie.TrieNode[1];
    
    public TrieBranch()
    {
      super();
    }
  }
  
  public class TrieNode
  {
    boolean isLeaf;
    int leafCount = 0;
    
    public TrieNode() {}
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\search\Trie.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */